@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # User Authentication Module
 #
-# $Id: lonauth.pm,v 1.89 2007/10/01 23:56:14 albertel Exp $
+# $Id: lonauth.pm,v 1.92 2008/05/30 19:09:50 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -37,6 +37,7 @@ use Crypt::DES;
 use Apache::loncommon();
 use Apache::lonnet;
 use Apache::lonmenu();
+use Apache::createaccount;
 use Fcntl qw(:flock);
 use Apache::lonlocal;
  
@@ -95,8 +96,8 @@ sub success {
 
     my %lt=&Apache::lonlocal::texthash(
 				       'wel' => 'Welcome',
-				       'mes' => 'Welcome to the Learning<i>Online</i> Network with CAPA. Please wait while your session is being set up',
-				       'pro' => 'Problems',
+				       'mes' => 'Welcome to the Learning<i>Online</i> Network with CAPA. Please wait while your session is being set up.',
+				       'pro' => 'Login problems?',
 				       'log' => 'loginproblems.html',
 				       );
     $r->print(<<ENDSUCCESS);
@@ -104,8 +105,8 @@ $start_page
 $setflags
 $windowinfo
 <h1>$lt{'wel'}</h1>
-$lt{'mes'}.<p>
-<a href="/adm/$lt{'log'}">$lt{'pro'}?</a></p>
+$lt{'mes'}<p>
+<a href="/adm/$lt{'log'}">$lt{'pro'}</a></p>
 $remoteinfo
 $maincall
 $continuelink
@@ -120,29 +121,19 @@ sub failed {
     my $start_page = &Apache::loncommon::start_page('Unsuccessful Login',undef,
 						    {'no_inline_link' => 1,});
     my $end_page   = &Apache::loncommon::end_page();
-
-    $message = &mt($message);
-    my %lt=('sorry'  => &mt('Sorry ...'),
-	    'please' => 
-	    &mt('Please [_1]log in again[_2].',
-		"<a href=\"/adm/login?username=$form->{'uname'}&domain=$form->{'udom'}\">",
-		'</a>'),
-	    'problemspage' => &mt('loginproblems.html'),
-	    'problems'     => 'Problems',
-	    );
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
-
-    $r->print(<<ENDFAILED);
-$start_page
-<h1>$lt{'sorry'}</h1>
-<p><b>$message</b></p>
-<p>$lt{'please'}</p>
-<p>
-<a href="/adm/$lt{'problemspage'}">$lt{'problems'}</a></p>
-$end_page
-ENDFAILED
-}
+    $r->print(
+       $start_page
+      .'<h1>'.&mt('Sorry ...').'</h1>'
+      .'<p><b>'.&mt($message).'</b></p>'
+      .'<p>'.&mt('Please [_1]log in again[_2].'
+                ,"<a href=\"/adm/login?username=$form->{'uname'}&domain=$form->{'udom'}\">",'</a>')
+      .'</p>'
+      .'<p><a href="/adm/loginproblems.html">'.&mt('Login problems?').'</a></p>'
+      .$end_page
+    );
+ }
 
 # ------------------------------------------------------------------ Rerouting!
 
@@ -150,8 +141,8 @@ sub reroute {
     my ($r) = @_;
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
-    my $msg='<h1>Sorry ...</h1>
-             Please <a href="/">log in again</a>.';
+    my $msg='<h1>'.&mt('Sorry ...').'</h1>'
+           .&mt('Please [_1]log in again[_2].');
     &Apache::loncommon::simple_error_page($r,'Rerouting',$msg);
 }
 
@@ -178,16 +169,16 @@ sub handler {
 	    &Apache::loncommon::start_page('Already logged in');
 	my $end_page = 
 	    &Apache::loncommon::end_page();
-	$r->print(<<ENDFAILED);
-$start_page
-<h1>You are already logged in</h1>
-<p>Please either <a href="/adm/roles">continue the current session</a> or
-<a href="/adm/logout">logout</a>.</p>
-<p>
-<a href="/adm/loginproblems.html">Problems?</a></p>
-$end_page
-ENDFAILED
-       return OK;
+        $r->print(
+           $start_page
+          .'<h1>'.&mt('You are already logged in!').'</h1>'
+          .'<p>'.&mt('Please either [_1]continue the current session[_2] or [_3]logout[_4].'
+                    ,'<a href="/adm/roles">','</a>','<a href="/adm/logout">','</a>')
+          .'</p>'
+          .'<p><a href="/adm/loginproblems.html">'.&mt('Login problems?').'</a></p>'
+          .$end_page
+        );
+        return OK;
     }
 
 # ---------------------------------------------------- No valid token, continue
@@ -263,9 +254,24 @@ ENDFAILED
     }
 
 # ---------------------------------------------------------------- Authenticate
-    my $authhost=Apache::lonnet::authenticate($form{'uname'},
-                                              $upass,
-                                              $form{'udom'});
+    my @cancreate;
+    my %domconfig = &Apache::lonnet::get_dom('configuration',['usercreation'],$form{'udom'});
+    if (ref($domconfig{'usercreation'}) eq 'HASH') {
+        if (ref($domconfig{'usercreation'}{'cancreate'}) eq 'HASH') {
+            if (ref($domconfig{'usercreation'}{'cancreate'}{'selfcreate'}) eq 'ARRAY') {
+                @cancreate = @{$domconfig{'usercreation'}{'cancreate'}{'selfcreate'}};
+            } elsif (($domconfig{'usercreation'}{'cancreate'}{'selfcreate'} ne 'none') && 
+                     ($domconfig{'usercreation'}{'cancreate'}{'selfcreate'} ne '')) {
+                @cancreate = ($domconfig{'usercreation'}{'cancreate'}{'selfcreate'});
+            }
+        }
+    }
+    my $defaultauth;
+    if (grep(/^login$/,@cancreate)) {
+        $defaultauth = 1;
+    }
+    my $authhost=Apache::lonnet::authenticate($form{'uname'},$upass,
+                                              $form{'udom'},$defaultauth);
     
 # --------------------------------------------------------------------- Failed?
 
@@ -273,6 +279,32 @@ ENDFAILED
 	&failed($r,'Username and/or password could not be authenticated.',
 		\%form);
         return OK;
+    } elsif ($authhost eq 'no_account_on_host') {
+        my %domconfig = 
+            &Apache::lonnet::get_dom('configuration',['usercreation'],$form{'udom'});
+        if (grep(/^login$/,@cancreate)) {
+            my $start_page = 
+                &Apache::loncommon::start_page('Create a user account in LON-CAPA',
+                                               '',{'no_inline_link'   => 1,});
+            my $domdesc = &Apache::lonnet::domain($form{'udom'},'description');
+            my ($output,$checkfail) = &Apache::createaccount::username_check($form{'uname'},
+                                                                  $form{'udom'},$domdesc);
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->send_http_header;
+            &Apache::createaccount::print_header($r,$start_page);
+            my $msg = '<h4>'.&mt('Although your username and password were authenticated, you do not currently have a LON-CAPA account in this domain.').'<br />';
+            if ($checkfail) {
+                $msg .= &mt('A LON-CAPA account may not be created with the username you used.');
+            } else {
+                $msg .= &mt('To create one, use the table below to provide information about yourself (if appropriate), then click the "Create LON-CAPA account" button.');
+            }
+            $r->print('<h4>'.$msg.'</h4>'.$output);
+            $r->print(&Apache::loncommon::end_page());
+            return OK;
+        } else {
+            &failed($r,'Although your username and password were authenticated, you do not currently have a LON-CAPA account in this domain, and you are not permitted to create one.',\%form);
+            return OK;
+        }
     }
 
     if (($firsturl eq '') || 
