@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Login Screen
 #
-# $Id: lonlogin.pm,v 1.100 2008/01/24 15:06:21 bisitz Exp $
+# $Id: lonlogin.pm,v 1.103 2008/05/30 19:31:20 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -47,6 +47,9 @@ sub handler {
 	      $ENV{'REDIRECT_QUERY_STRING'}),
 	 ['interface','username','domain','firsturl','localpath','localres',
 	  'token']);
+    if (!defined($env{'form.firsturl'})) {
+        &Apache::lonacc::get_posted_cgi($r,['firsturl']);
+    }
 
 # -- check if they are a migrating user
     if (defined($env{'form.token'})) {
@@ -80,7 +83,7 @@ sub handler {
 	    &Apache::loncommon::end_page();
 	$r->print(
                   $start_page
-                 .'<h1>'.&mt('You are already logged in').'</h1>'
+                 .'<h1>'.&mt('You are already logged in!').'</h1>'
                  .'<p>'.&mt('Please either [_1]continue the current session[_2] or [_3]logout[_4].',
                   '<a href="/adm/roles">','</a>','<a href="/adm/logout">','</a>').'</p>'
                  .'<p><a href="/adm/loginproblems.html">'.&mt('Login problems?').'</a></p>'
@@ -218,16 +221,20 @@ sub handler {
                 $hostname.'</a>';
 	    $last=$hostname;
         }
-	$r->print(<<ENDTROUBLE);
-<html>
-<head><title>The LearningOnline Network with CAPA</title></head>
-<body bgcolor="#FFFFFF">
-<img src="/adm/lonKaputt/lonlogo_broken.gif" align="right" />
-<h3>This LON-CAPA server is temporarily not available for login</h3>
-<p>Please attempt to login to one of the following servers:</p>$spares
-</body>
-</html>
-ENDTROUBLE
+	$r->print(
+           '<html>'
+          .'<head><title>'
+          .&mt('The LearningOnline Network with CAPA')
+          .'</title></head>'
+          .'<body bgcolor="#FFFFFF">'
+          .'<h1>'.&mt('The LearningOnline Network with CAPA').'</h1>'
+          .'<img src="/adm/lonKaputt/lonlogo_broken.gif" align="right" />'
+          .'<h3>'.&mt('This LON-CAPA server is temporarily not available for login.').'</h3>'
+          .'<p>'.&mt('Please attempt to login to one of the following servers:').'</p>'
+          .$spares
+          .'</body>'
+          .'</html>'
+        );
         return OK;
     }
 
@@ -263,6 +270,8 @@ ENDTROUBLE
     my $showcoursecat =
         &Apache::loncommon::designparm('login.coursecatalog',$domain);
     my $loginheader =&Apache::loncommon::designparm('login.loginheader',$domain);
+    my $shownewuserlink = 
+        &Apache::loncommon::designparm('login.newuser',$domain);
     my $now=time;
     my $js = (<<ENDSCRIPT);
 
@@ -349,6 +358,7 @@ my %lt=&Apache::lonlocal::texthash(
                   'servadm' => 'Server Administration',
                   'helpdesk' => 'Contact Helpdesk',
                   'forgotpw' => 'Forgot password?',
+                  'newuser'  => 'New User?',
                   'options_headline' => 'Select Accessibility Options',
                   'sprs_img' => 'Suppress rendering of images',
                   'sprs_applet' => 'Suppress Java applets',
@@ -396,6 +406,10 @@ ENDSERVERFORM
     if (($showcoursecat eq '') || ($showcoursecat)) {
         $coursecatalog = &coursecatalog_link($lt{'catalog'});
     }
+    my $newuserlink;
+    if ($shownewuserlink) {
+        $newuserlink = &newuser_link($lt{'newuser'});
+    }
     if ($fullgraph) { $r->print(<<ENDTOP);
   <!-- The LON-CAPA Header -->
   <tr>
@@ -523,6 +537,8 @@ ENDNOOPT
        <td bgcolor="$mainbg" valign="bottom" align="left" colspan="2">
         $loginhelp
         $forgotpw
+        $newuserlink
+        <br />
        </td>
       </tr>
      </table>
@@ -667,7 +683,7 @@ sub forgotpwdisplay {
     my (%lt) = @_;
     my $prompt_for_resetpw = 1; 
     if ($prompt_for_resetpw) {
-        return '<br />&nbsp;&nbsp;&nbsp;<a href="/adm/resetpw">'.$lt{'forgotpw'}.'</a></b><br /><br />';
+        return '<br />&nbsp;&nbsp;&nbsp;<a href="/adm/resetpw">'.$lt{'forgotpw'}.'</a></b><br />';
     }
     return;
 }
@@ -691,5 +707,10 @@ sub coursecatalog_link {
 END
 }
 
+sub newuser_link {
+    my ($linkname) = @_;
+    return '&nbsp;&nbsp;&nbsp;<a href="/adm/createaccount"><b>'.$linkname.'</b></a><br />';
+}
+
 1;
 __END__
