@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # User Roles Screen
 #
-# $Id: lonroles.pm,v 1.181.2.2 2008/03/22 03:52:04 raeburn Exp $
+# $Id: lonroles.pm,v 1.200 2008/07/08 20:16:16 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -106,29 +106,67 @@ sub handler {
 
 # ================================================================== Roles Init
     if ($env{'form.selectrole'}) {
+
+        my $locknum=&Apache::lonnet::get_locks();
+        if ($locknum) { return 409; }
+
         if ($env{'form.newrole'}) {
             $env{'form.'.$env{'form.newrole'}}=1;
 	}
 	if ($env{'request.course.id'}) {
+            # Check if user is CC trying to select a course role
+            if ($env{'form.switchrole'}) {
+                if (!defined($env{'user.role.'.$env{'form.switchrole'}})) {
+                    &adhoc_course_role($then);
+                }
+            }
 	    my %temp=('logout_'.$env{'request.course.id'} => time);
 	    &Apache::lonnet::put('email_status',\%temp);
 	    &Apache::lonnet::delenv('user.state.'.$env{'request.course.id'});
 	}
-	&Apache::lonnet::appenv("request.course.id"   => '',
-				"request.course.fn"   => '',
-				"request.course.uri"  => '',
-				"request.course.sec"  => '',
-				"request.role"        => 'cm',
-                                "request.role.adv"    => $env{'user.adv'},
-				"request.role.domain" => $env{'user.domain'});
-
-# Check if user is a DC trying to enter a course and needs privs to be created
+	&Apache::lonnet::appenv({"request.course.id"   => '',
+			 	 "request.course.fn"   => '',
+				 "request.course.uri"  => '',
+				 "request.course.sec"  => '',
+				 "request.role"        => 'cm',
+                                 "request.role.adv"    => $env{'user.adv'},
+				 "request.role.domain" => $env{'user.domain'}});
+# Check if user is a DC trying to enter a course or author space and needs privs to be created
         if ($numdc > 0) {
             foreach my $envkey (keys %env) {
+# Is this an ad-hoc CC-role?
                 if (my ($domain,$coursenum) =
 		    ($envkey =~ m-^form\.cc\./($match_domain)/($match_courseid)$-)) {
                     if ($dcroles{$domain}) {
-                        &check_privs($domain,$coursenum,$then,$now);
+                        &check_privs($domain,$coursenum,$then,$now,'cc');
+                    }
+                    last;
+                }
+# Is this an ad-hoc CA-role?
+                if (my ($domain,$user) =
+		    ($envkey =~ m-^form\.ca\./($match_domain)/($match_username)$-)) {
+                   # Check if author blocked ca-access
+                    my %blocked=&Apache::lonnet::get('environment',['domcoord.author'],$domain,$user);
+                    if ($blocked{'domcoord.author'} eq 'blocked') {
+                       delete($env{$envkey});
+                       $env{'user.error.msg'}=':::1:User '.$user.' in domain '.$domain.' blocked domain coordinator access';
+                       last;
+                    }
+                    if ($dcroles{$domain}) {
+                        my ($server_status,$home) = &check_author_homeserver($user,$domain);
+                        if (($server_status eq 'ok') || ($server_status eq 'switchserver')) {
+                            &check_privs($domain,$user,$then,$now,'ca');
+                            if ($server_status eq 'switchserver') {
+                                my $trolecode = 'ca./'.$domain.'/'.$user; 
+                                my $switchserver = '/adm/switchserver?'
+                                                  .'otherserver='.$home.'&role='.$trolecode;
+                                $r->internal_redirect($switchserver);
+                            }
+                        } else {
+                            delete($env{$envkey});
+                        }
+                    } else {
+                        delete($env{$envkey});
                     }
                     last;
                 }
@@ -245,10 +283,10 @@ ENDENTERKEY
 					 "Role ".$trolecode);
 		    
 		    &Apache::lonnet::appenv(
-					   'request.role'        => $trolecode,
-					   'request.role.domain' => $cdom,
-					   'request.course.sec'  => $csec,
-                                           'request.course.groups' => $cgrps);
+					   {'request.role'        => $trolecode,
+					    'request.role.domain' => $cdom,
+					    'request.course.sec'  => $csec,
+                                            'request.course.groups' => $cgrps});
                     my $tadv=0;
 
 		    if (($cnum) && ($role ne 'ca') && ($role ne 'aa')) {
@@ -259,7 +297,7 @@ ENDENTERKEY
 			    ($env{'form.orgurl'}!~/^\/adm\/flip/)) {
 			    my $dest=$env{'form.orgurl'};
 			    if (&Apache::lonnet::allowed('adv') eq 'F') { $tadv=1; }
-			    &Apache::lonnet::appenv('request.role.adv'=>$tadv);
+			    &Apache::lonnet::appenv({'request.role.adv'=>$tadv});
                             if (($ferr) && ($tadv)) {
 				&error_page($r,$ferr,$dest);
 			    } else {
@@ -269,7 +307,7 @@ ENDENTERKEY
 			} else {
 			    if (!$env{'request.course.id'}) {
 				&Apache::lonnet::appenv(
-				      "request.course.id"  => $cdom.'_'.$cnum);
+				      {"request.course.id"  => $cdom.'_'.$cnum});
 				$furl='/adm/roles?tryagain=1';
 				$msg=
 				    '<h1><span class="LC_error">'.
@@ -278,7 +316,7 @@ ENDENTERKEY
 				    '</span></h1><h3>'.&mt('Please try again.').'</h3>'.$ferr;
 			    }
 			    if (&Apache::lonnet::allowed('adv') eq 'F') { $tadv=1; }
-			    &Apache::lonnet::appenv('request.role.adv'=>$tadv);
+			    &Apache::lonnet::appenv({'request.role.adv'=>$tadv});
 
 			    if (($ferr) && ($tadv)) {
 				&error_page($r,$ferr,$furl);
@@ -295,6 +333,14 @@ ENDENTERKEY
 				    $furl = "/adm/helper/course.initialization.helper";
 				    # Send the user to the course they selected
 				} elsif ($env{'request.course.id'}) {
+                                    if ($env{'form.destinationurl'}) {
+                                        my $dest = $env{'form.destinationurl'};
+                                        &redirect_user($r,&mt('Entering [_1]',
+                                                      $env{'course.'.$courseid.'.description'}),
+                                               $dest,$msg,
+                                               $env{'environment.remotenavmap'});
+                                        return OK;
+                                    }
 				    if (&Apache::lonnet::allowed('whn',
 								 $env{'request.course.id'})
 					|| &Apache::lonnet::allowed('whn',
@@ -455,17 +501,14 @@ ENDHEADER
         $r->print('<input type="hidden" name="selectrole" value="1" />');
         $r->print('<input type="hidden" name="newrole" value="" />');
     }
-    if ($env{'user.adv'}) {
-	$r->print(
-	      '<br /><span class="LC_rolesinfo"><label>'.&mt('Show all roles').': <input type="checkbox" name="showall"');
-	if ($env{'form.showall'}) { $r->print(' checked="checked" '); }
-	$r->print(' /></label><input type="submit" value="'.&mt('Display').'" /></span>');
-    }
-
     my (%roletext,%sortrole,%roleclass);
     my $countactive=0;
+    my $countfuture=0;
+    my $countwill=0;
     my $inrole=0;
     my $possiblerole='';
+    my %futureroles;
+    my %roles_nextlogin;
     foreach $envkey (sort keys %env) {
         my $button = 1;
         my $switchserver='';
@@ -502,9 +545,13 @@ ENDHEADER
                 } elsif ($tstatus eq 'future') {
                     $tbg='#FFFF77';
                     $button=0;
+                    $futureroles{$trolecode} = $tstart.':'.$tend;
+                    $countfuture ++;
                 } elsif ($tstatus eq 'will') {
                     $tbg='#FFAA77';
                     $tremark.=&mt('Active at next login. ');
+                    $roles_nextlogin{$trolecode} = $tstart.':'.$tend;
+                    $countwill ++;
                 } elsif ($tstatus eq 'expired') {
                     $tbg='#FF7777';
                     $tfont='#330000';
@@ -620,6 +667,18 @@ ENDHEADER
 	    }
         }
     }
+    if ($env{'user.adv'}) {
+        $r->print(
+              '<br /><span class="LC_rolesinfo"><label>'.&mt('Show all roles').': <input type="checkbox" name="showall"');
+        if ($env{'form.showall'}) { $r->print(' checked="checked" '); }
+        $r->print(' /></label><input type="submit" value="'.&mt('Display').'" /></span>');
+    } else {
+        if ($countactive > 0) {
+            my $domdesc = &Apache::lonnet::domain($env{'user.domain'},'description');
+            $r->print('<p>'.&mt('[_1]Visit the [_2]Course Catalog[_3] to view all [_4] LON-CAPA courses.','<b>','<a href="/adm/coursecatalog">','</a></b>',$domdesc).'<br />'.&mt('If a course is [_1]not[_2] in your list of current courses below, you may be able to enroll if self-enrollment is permitted.','<b>','</b>').'</p>');
+        }
+    }
+
 # No active roles
     if ($countactive==0) {
 	if ($inrole) {
@@ -627,7 +686,30 @@ ENDHEADER
 	} else {
 	    $r->print('<h2>'.&mt('Currently no active roles or courses').'</h2>');
 	}
-	$r->print('</form>'.&Apache::loncommon::end_page());
+        &findcourse_advice($r);
+	$r->print('</form>');
+        if ($countfuture) {
+            $r->print(&mt('The following [quant,_1,role,roles] will become active in the future:',$countfuture));
+            my $doheaders = &roletable_headers($r,\%roleclass,\%sortrole,
+                                               $nochoose);
+            &print_rolerows($r,$doheaders,\%roleclass,\%sortrole,\%dcroles,
+                            \%roletext);
+            my $tremark='';
+            my $tfont='#003300';
+            if ($env{'request.role'} eq 'cm') {
+                $r->print('<tr bgcolor="#11CC55">');
+                $tremark=&mt('Currently selected. ');
+                $tfont='#002200';
+            } else {
+                $r->print('<tr bgcolor="#77FF77">');
+            }
+            $r->print('<td></td><td colspan="3"><font color="'.$tfont.'"><span class="LC_rolesinfo">'.&mt('No role specified').
+                      '</span></font></td><td><font color="'.$tfont.'">'.$tremark.
+                      '&nbsp;</font></td></tr>'."\n");
+
+            $r->print('</table>');
+        }
+        $r->print(&Apache::loncommon::end_page());
 	return OK;
 # Is there only one choice?
     } elsif (($countactive==1) && ($env{'request.role'} eq 'cm')) {
@@ -645,21 +727,7 @@ ENDHEADER
     unless ((!&Apache::lonmenu::show_course()) || ($nochoose)) {
 	$r->print("<h2>".&mt('Select a Course to Enter')."</h2>\n");
     }
-    $r->print('<br /><table id="LC_rolesmenu"><tr>');
-    unless ($nochoose) { $r->print('<th>&nbsp;</th>'); }
-    $r->print('<th>'.&mt('User Role').'</th><th>'.&mt('Extent').
-         '</th><th>'.&mt('Start').'</th><th>'.&mt('End').'</th></tr>'."\n");
-    my $doheaders=-1;
-    foreach my $type ('Domain','Construction Space','Course','Unavailable','System') {
-	my $haverole=0;
-	foreach my $which (sort {uc($a) cmp uc($b)} (keys(%sortrole))) {
-	    if ($roleclass{$sortrole{$which}} =~ /^\Q$type\E/) { 
-		$haverole=1;
-	    }
-	}
-	if ($haverole) { $doheaders++; }
-    }
-
+    my $doheaders = &roletable_headers($r,\%roleclass,\%sortrole,$nochoose);
     if ($env{'environment.recentroles'}) {
         my %recent_roles =
                &Apache::lonhtmlcommon::get_recent('roles',$env{'environment.recentrolesn'});
@@ -669,7 +737,7 @@ ENDHEADER
 		$output.=$roletext{'user.role.'.$_};
                 if ($_ =~ m-dc\./($match_domain)/- 
 		    && $dcroles{$1}) {
-		    $output .= &allcourses_row($1,'recent');
+		    $output .= &adhoc_roles_row($1,'recent');
                 }
 	    } elsif ($numdc > 0) {
                 unless ($_ =~/^error\:/) {
@@ -679,37 +747,18 @@ ENDHEADER
 	}
 	if ($output) {
 	    $r->print("<tr><td align='center' colspan='5'><font face='arial'>".
-		      &mt('Recent Roles')."</font></td>");
+		      &mt('Recent Roles')."</font></td></tr>");
 	    $r->print($output);
-	    $r->print("</tr>");
             $doheaders ++;
 	}
     }
 
     if ($numdc > 0) {
         $r->print(&coursepick_jscript());
-        $r->print(&Apache::loncommon::coursebrowser_javascript());
-    }
-    foreach my $type ('Construction Space','Domain','Course','Unavailable','System') {
-	my $output;
-	foreach my $which (sort {uc($a) cmp uc($b)} (keys(%sortrole))) {
-	    if ($roleclass{$sortrole{$which}} =~ /^\Q$type\E/) { 
-		$output.=$roletext{$sortrole{$which}};
-                if ($sortrole{$which} =~ m-dc\./($match_domain)/-) {
-                    if ($dcroles{$1}) {
-                        $output .= &allcourses_row($1,'');
-                    }
-                }
-	    }
-	}
-	if ($output) {
-	    if ($doheaders > 0) {
-		$r->print("<tr>".
-			  "<td align='center' colspan='5'><font face='arial'>".&mt($type)."</font></td></tr>");
-	    }
-	    $r->print($output);	
-	}
+        $r->print(&Apache::loncommon::coursebrowser_javascript().
+                  &Apache::loncommon::authorbrowser_javascript());
     }
+    &print_rolerows($r,$doheaders,\%roleclass,\%sortrole,\%dcroles,\%roletext);
     my $tremark='';
     my $tfont='#003300';
     if ($env{'request.role'} eq 'cm') {
@@ -722,13 +771,13 @@ ENDHEADER
     unless ($nochoose) {
 	if ($env{'request.role'} ne 'cm') {
 	    $r->print('<td><input type="submit" value="'.
-		      &mt('Select').'" name="cm"></td>');
+		      &mt('Select').'" name="cm" /></td>');
 	} else {
 	    $r->print('<td>&nbsp;</td>');
 	}
     }
     $r->print('<td colspan="3"><font color="'.$tfont.'"><span class="LC_rolesinfo">'.&mt('No role specified').
-      '</font></span></td><td><font color="'.$tfont.'">'.$tremark.
+      '</span></font></td><td><font color="'.$tfont.'">'.$tremark.
       '&nbsp;</font></td></tr>'."\n");
 
     $r->print('</table>');
@@ -742,14 +791,98 @@ ENDHEADER
     }
     $r->print(&Apache::lonnet::getannounce());
     if ($advanced) {
-	$r->print('<p><small><i>This is LON-CAPA '.
-		  $r->dir_config('lonVersion').'</i><br />'.
-		  '<a href="/adm/logout">'.&mt('Logout').'</a></small></p>');
+	$r->print('<p><small><i>'
+                 .&mt('This is LON-CAPA [_1]',$r->dir_config('lonVersion'))
+		 .'</i><br />'
+		 .'<a href="/adm/logout">'.&mt('Logout').'</a>&nbsp;&nbsp;'
+                 .'<a href="/adm/coursecatalog">'.&mt('Course Catalog')
+                 .'</small></p>');
     }
     $r->print(&Apache::loncommon::end_page());
     return OK;
 }
 
+sub roletable_headers {
+    my ($r,$roleclass,$sortrole,$nochoose) = @_;
+    my $doheaders;
+    if ((ref($sortrole) eq 'HASH') && (ref($roleclass) eq 'HASH')) {
+        $r->print('<br /><table id="LC_rolesmenu"><tr>');
+        if (!$nochoose) { $r->print('<th>&nbsp;</th>'); }
+        $r->print('<th>'.&mt('User Role').'</th><th>'.&mt('Extent')
+                  .'</th><th>'.&mt('Start').'</th><th>'.&mt('End')
+                  .'</th></tr>'."\n");
+        $doheaders=-1;
+        my @roletypes = &roletypes();
+        foreach my $type (@roletypes) {
+            my $haverole=0;
+            foreach my $which (sort {uc($a) cmp uc($b)} (keys(%{$sortrole}))) {
+                if ($roleclass->{$sortrole->{$which}} =~ /^\Q$type\E/) {
+                    $haverole=1;
+                }
+            }
+            if ($haverole) { $doheaders++; }
+        }
+    }
+    return $doheaders;
+}
+
+sub roletypes {
+    my @types = ('Domain','Construction Space','Course','Unavailable','System');
+    return @types; 
+}
+
+sub print_rolerows {
+    my ($r,$doheaders,$roleclass,$sortrole,$dcroles,$roletext) = @_;
+    if ((ref($roleclass) eq 'HASH') && (ref($sortrole) eq 'HASH')) {
+        my @types = &roletypes();
+        foreach my $type (@types) {
+            my $output;
+            foreach my $which (sort {uc($a) cmp uc($b)} (keys(%{$sortrole}))) {
+                if ($roleclass->{$sortrole->{$which}} =~ /^\Q$type\E/) {
+                    if (ref($roletext) eq 'HASH') {
+                        $output.=$roletext->{$sortrole->{$which}};
+                        if ($sortrole->{$which} =~ m-dc\./($match_domain)/-) {
+                            if (ref($dcroles) eq 'HASH') {
+                                if ($dcroles->{$1}) {
+                                    $output .= &adhoc_roles_row($1,'');
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            if ($output) {
+                if ($doheaders > 0) {
+                    $r->print("<tr>".
+                              "<td align='center' colspan='5'><font face='arial'>".
+                              &mt($type)."</font></td></tr>");
+                }
+                $r->print($output);
+            }
+        }
+    }
+}
+
+sub findcourse_advice {
+    my ($r) = @_;
+    my $domdesc = &Apache::lonnet::domain($env{'user.domain'},'description');
+    if (&Apache::lonnet::auto_run(undef,$env{'user.domain'})) {
+        $r->print(&mt('If you were expecting to see an active role listed for a particular course in the [_1] domain, it may be missing for one of the following reasons:',$domdesc).'
+<ul>
+ <li>'.&mt('The course has yet to be created.').'</li>
+ <li>'.&mt('Automatic enrollment of registered students has not been enabled for the course.').'</li>
+ <li>'.&mt('You are in a section of course for which automatic enrollment in the corresponding LON-CAPA course is not active.').'</li>
+ <li>'.&mt('The start date for automated enrollment has yet to be reached.').'</li>
+ <li>'.&mt('You registered for the course recently and there is a time lag between the time you register, and the time this information becomes available for the update of LON-CAPA course rosters.').'</li>
+ </ul>');
+    } else {
+        $r->print(&mt('If you were expecting to see an active role listed for a particular course, that course may not have been created yet.').'<br />');
+    }
+    $r->print('<p>'.&mt('The [_1]Course Catalog[_2] provides information about all [_3] classes for which LON-CAPA courses have been created.','<a href="/adm/coursecatalog">','</a>',$domdesc).'<br />');
+    $r->print(&mt('You can search the course catalog for courses which permit self-enrollment, if you would like to enroll in a course.').'</p>');
+    return;
+}
+
 sub privileges_info {
     my ($which) = @_;
     my $output;
@@ -850,17 +983,17 @@ sub build_roletext {
         } elsif ($tstatus eq 'is') {
             $roletext.='<td'.$rowspan.'><input name="'.$buttonname.'" type="button" value="'.
                         &mt('Select').'" onClick="javascript:enterrole(this.form,\''.
-                        $trolecode."','".$buttonname.'\');"></td>';
+                        $trolecode."','".$buttonname.'\');" /></td>';
         } elsif ($tryagain) {
             $roletext.=
                 '<td'.$rowspan.'><input name="'.$buttonname.'" type="button" value="'.
                 &mt('Try Selecting Again').'" onClick="javascript:enterrole(this.form,\''.
-                        $trolecode."','".$buttonname.'\');"></td>';
+                        $trolecode."','".$buttonname.'\');" /></td>';
         } elsif ($advanced) {
             $roletext.=
                 '<td'.$rowspan.'><input name="'.$buttonname.'" type="button" value="'.
                 &mt('Re-Initialize').'" onClick="javascript:enterrole(this.form,\''.
-                        $trolecode."','".$buttonname.'\');"></td>';
+                        $trolecode."','".$buttonname.'\');" /></td>';
         } else {
             $roletext.='<td'.$rowspan.'>&nbsp;</td>';
         }
@@ -881,17 +1014,34 @@ sub build_roletext {
     return $roletext;
 }
 
+sub check_author_homeserver {
+    my ($uname,$udom)=@_;
+    if (($uname eq '') || ($udom eq '')) {
+        return ('fail','');
+    }
+    my $home = &Apache::lonnet::homeserver($uname,$udom);
+    if (&Apache::lonnet::host_domain($home) ne $udom) {
+        return ('fail',$home);
+    }
+    my @ids=&Apache::lonnet::current_machine_ids();
+    if (grep(/^\Q$home\E$/,@ids)) {
+        return ('ok',$home);
+    } else {
+        return ('switchserver',$home);
+    }
+}
+
 sub check_privs {
-    my ($cdom,$cnum,$then,$now) = @_;
-    my $cckey = 'user.role.cc./'.$cdom.'/'.$cnum; 
+    my ($cdom,$cnum,$then,$now,$checkrole) = @_;
+    my $cckey = 'user.role.'.$checkrole.'./'.$cdom.'/'.$cnum; 
     if ($env{$cckey}) {
         my ($role,$where,$trolecode,$tstart,$tend,$tremark,$tstatus,$tpstart,$tpend,$tfont);
         &role_status($cckey,$then,$now,\$role,\$where,\$trolecode,\$tstatus,\$tstart,\$tend);
         unless (($tstatus eq 'is') || ($tstatus eq 'will_not')) {
-            &set_privileges($cdom,$cnum);
+            &set_privileges($cdom,$cnum,$checkrole);
         }
     } else {
-        &set_privileges($cdom,$cnum);
+        &set_privileges($cdom,$cnum,$checkrole);
     }
 }
 
@@ -916,8 +1066,71 @@ sub check_fordc {
     return $numdc;
 }
 
+sub adhoc_course_role {
+    my ($then) = @_; 
+    my ($cdom,$cnum);
+    my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+    my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+    if (&check_forcc($cdom,$cnum,$then)) {
+        my $setprivs;
+        if (!defined($env{'user.role.'.$env{'form.switchrole'}})) {
+            $setprivs = 1;
+        } else {
+            my ($start,$end) = split(/\./,$env{'user.role.'.$env{'form.switchrole'}});
+            if (($start && ($start>$then || $start == -1)) ||
+                ($end && $end<$then)) {
+                $setprivs = 1;
+            }
+        } 
+        if ($setprivs) {
+            if ($env{'form.switchrole'} =~ m-^(in|ta|ep|ad|st|cr)([\w/]*)\./\Q$cdom\E/\Q$cnum\E/?(\w*)$-) {
+                my $role = $1;
+                my $custom_role = $2;
+                my $usec = $3;
+                if ($role eq 'cr') {
+                    if ($custom_role =~ m-^/$match_domain/$match_username/\w+$-) {
+                        $role .= $custom_role;
+                    } else {
+                        return;
+                    }
+                }
+                my (%userroles,%newrole,%newgroups);
+                my $area = '/'.$cdom.'/'.$cnum;
+                my $spec = $role.'.'.$area;
+                if ($usec ne '') {
+                    $spec .= '/'.$usec;
+                    $area .= '/'.$usec;
+                }
+                &Apache::lonnet::standard_roleprivs(\%newrole,$role,$cdom,$spec,$cnum,$area);
+                &Apache::lonnet::set_userprivs(\%userroles,\%newrole,%newgroups);
+                my $adhocstart = $then-1;
+                $userroles{'user.role.'.$spec} = $adhocstart.'.';
+                &Apache::lonnet::appenv(\%userroles,[$role,'cm']);
+            }
+        }
+    }
+    return;
+}
+
+sub check_forcc {
+    my ($cdom,$cnum,$then) = @_;
+    my $is_cc;
+    if ($cdom ne '' && $cnum ne '') {
+        if (&Apache::lonnet::is_course($cdom,$cnum)) {
+            my $envkey = 'user.role.cc./'.$cdom.'/'.$cnum;
+            if (defined($env{$envkey})) {
+                $is_cc = 1;
+                my ($tstart,$tend)=split(/\./,$env{$envkey});
+                if ($tstart && $tstart>$then) { $is_cc = 0; }
+                if ($tend   && $tend  <$then) { $is_cc = 0; }
+            }
+        }
+    }
+    return $is_cc;
+}
+
 sub courselink {
-    my ($dcdom,$rowtype,$selecttype) = @_;
+    my ($dcdom,$rowtype) = @_;
     my $courseform=&Apache::loncommon::selectcourse_link
                    ('rolechoice','dccourse'.$rowtype.'_'.$dcdom,
                     'dcdomain'.$rowtype.'_'.$dcdom,'coursedesc'.$rowtype.'_'.
@@ -933,7 +1146,7 @@ sub coursepick_jscript {
     my %lt = &Apache::lonlocal::texthash(
                   plsu => "Please use the 'Select Course' link to open a separate pick course window where you may select the course you wish to enter.",
                   youc => 'You can only use this screen to select courses in the current domain.',
-            );
+             );
     my $verify_script = <<"END";
 <script type="text/javascript">
 function verifyCoursePick(caller) {
@@ -953,7 +1166,7 @@ function verifyCoursePick(caller) {
         }
     }
     else {
-        alert("$lt{'youc'}");
+        alert("$lt{'youc'}")
     }
 }
 function getIndex(caller) {
@@ -969,6 +1182,13 @@ END
     return $verify_script;
 }
 
+sub coauthorlink {
+    my ($dcdom,$rowtype) = @_;
+    my $coauthorform=&Apache::loncommon::selectauthor_link('rolechoice',$dcdom);
+    my $hiddenitems = '<input type="hidden" name="adhoccauname'.$rowtype.'_'.$dcdom.'" value="" />';
+    return $coauthorform.$hiddenitems;
+}
+
 sub display_cc_role {
     my $rolekey = shift;
     my $roletext;
@@ -1001,15 +1221,22 @@ sub display_cc_role {
     return ($roletext);
 }
 
-sub allcourses_row {
+sub adhoc_roles_row {
     my ($dcdom,$rowtype) = @_;
     my $output = '<tr bgcolor="#77FF77">'.
-                 ' <td colspan="5">';
-    my $selectlink = &courselink($dcdom,$rowtype);
+                 ' <td colspan="5"><table><tr><td><span class="LC_rolesinfo">'
+                 .&mt('[_1]Ad hoc[_2] roles in domain [_3] --',
+                 '<span class="LC_cusr_emph">','</span>',$dcdom).'</span></td><td>';
+    my $selectcclink = &courselink($dcdom,$rowtype);
     my $ccrole = &Apache::lonnet::plaintext('cc');
+    my $carole = &Apache::lonnet::plaintext('ca');
+    my $selectcalink = &coauthorlink($dcdom,$rowtype);
     $output.= '<span class="LC_rolesinfo">'.
-            &mt('[_1]: [_2] from domain [_3]',$ccrole,$selectlink,$dcdom).
-            '</span><br /></tr><tr><td colspan="5" height="3"></td></tr>'."\n";
+            &mt('[_1]: [_2]',$ccrole,$selectcclink).
+            '</span><br /></td><td>&nbsp;&nbsp;</td><td><span class="LC_rolesinfo">'.
+            &mt('[_1]: [_2]',$carole,$selectcalink).
+            '</span><br /></td></tr></table></td></tr>'.
+            '<tr><td colspan="5" height="3"></td></tr>'."\n";
     return $output;
 }
 
@@ -1019,9 +1246,9 @@ sub recent_filename {
 }
 
 sub set_privileges {
-    my ($dcdom,$pickedcourse) = @_;
+# role can be cc or ca
+    my ($dcdom,$pickedcourse,$role) = @_;
     my $area = '/'.$dcdom.'/'.$pickedcourse;
-    my $role = 'cc';
     my $spec = $role.'.'.$area;
     my %userroles = &Apache::lonnet::set_arearole($role,$area,'','',
 						  $env{'user.domain'},
@@ -1029,18 +1256,19 @@ sub set_privileges {
     my %ccrole = ();
     &Apache::lonnet::standard_roleprivs(\%ccrole,$role,$dcdom,$spec,$pickedcourse,$area);
     my ($author,$adv)= &Apache::lonnet::set_userprivs(\%userroles,\%ccrole);
-    &Apache::lonnet::appenv(%userroles);
+    &Apache::lonnet::appenv(\%userroles,[$role,'cm']);
+
     &Apache::lonnet::log($env{'user.domain'},
                          $env{'user.name'},
                          $env{'user.home'},
                         "Role ".$role);
     &Apache::lonnet::appenv(
-                          'request.role'        => $spec,
+                          {'request.role'        => $spec,
                           'request.role.domain' => $dcdom,
-                          'request.course.sec'  => '');
+                          'request.course.sec'  => ''});
     my $tadv=0;
     if (&Apache::lonnet::allowed('adv') eq 'F') { $tadv=1; }
-    &Apache::lonnet::appenv('request.role.adv'    => $tadv);
+    &Apache::lonnet::appenv({'request.role.adv'    => $tadv});
 }
 
 sub courseloadpage {
