@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Cookie Based Access Handler
 #
-# $Id: publiccheck.pm,v 1.14 2007/04/11 21:36:58 raeburn Exp $
+# $Id: publiccheck.pm,v 1.15 2007/10/02 01:09:59 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -34,7 +34,6 @@ use Apache::Constants qw(:common :http :methods);
 use Apache::lonnet;
 use Apache::loncommon();
 use Apache::lonlocal;
-use CGI::Cookie();
 use Fcntl qw(:flock);
 use Apache::lonacc();
 use LONCAPA();
@@ -46,17 +45,14 @@ sub handler {
     if (&Apache::lonnet::is_domainimage($requrl)) {
         return OK;
     }
-    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
-    my $lonid=$cookies{'lonID'};
-    if ($lonid) {
-	my $handle=&LONCAPA::clean_handle($lonid->value);
+
+    my $handle = &Apache::lonnet::check_for_valid_session($r);
+    if ($handle ne '') {
         my $lonidsdir=$r->dir_config('lonIDsDir');
-        if ((-e "$lonidsdir/$handle.id") && ($handle ne '')) {
-	    &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);
-	    if ($env{'user.name'} ne 'public'
-		&& $env{'user.domain'} ne 'public') {
-		return OK;
-	    }
+	&Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);
+	if ($env{'user.name'} ne 'public'
+	    && $env{'user.domain'} ne 'public') {
+	    return OK;
 	}
     }
     if ($requrl=~m|^/public/|
