@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA 
 # edit mode helpers
 #
-# $Id: edit.pm,v 1.113 2008/01/15 03:09:09 raeburn Exp $
+# $Id: edit.pm,v 1.114 2008/03/12 02:46:52 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -880,7 +880,7 @@ sub entercoords {
 	$data{"imagechoice.$id.formwidth"}=$width.'_'.
 	    $Apache::edit::bgimgsrccurdepth;
     }
-    &Apache::lonnet::appenv(%data);
+    &Apache::lonnet::appenv(\%data);
     my $text="Click Coordinates";
     my $result='<a href="/adm/imagechoice?token='.$id.'" target="imagechoice">'.$text.'</a>';
     return $result;
@@ -913,7 +913,7 @@ sub entercoord {
 	$data{"imagechoice.$id.formwidth"}=$width.'_'.
 	    $Apache::edit::bgimgsrccurdepth;
     }
-    &Apache::lonnet::appenv(%data);
+    &Apache::lonnet::appenv(\%data);
     my $text="Enter Coordinates";
     if ($type eq 'polygon') { $text='Create Polygon Data'; }
     my $result='<a href="/adm/imagechoice?token='.$id.'" target="imagechoice">'.$text.'</a>';
