@@ -2,7 +2,7 @@
 # The LearningOnline Network with CAPA
 # image click response style
 #
-# $Id: imageresponse.pm,v 1.92 2008/02/01 22:05:45 raeburn Exp $
+# $Id: imageresponse.pm,v 1.93 2008/03/12 02:46:53 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -223,7 +223,7 @@ sub draw_image {
 	    }
 	}
     }
-    &Apache::lonnet::appenv(%x);
+    &Apache::lonnet::appenv(\%x);
     return $id;
 }
 
