@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # input  definitons
 #
-# $Id: inputtags.pm,v 1.242 2008/03/17 15:40:31 bisitz Exp $
+# $Id: inputtags.pm,v 1.243 2008/05/22 22:04:31 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -729,7 +729,7 @@ sub decideoutput {
 	$css_class=$possible_class{'not_charged_try'};
 	$button = 1;
     } elsif ($award eq 'MISSING_ANSWER') {
-	$message = &mt('Some items were not submitted.');
+	$message = &mt('Some items were not submitted.').&Apache::loncommon::help_open_topic('Some_Items_Were_Not_Submitted');
 	$css_class=$possible_class{'not_charged_try'};
 	$button = 1;
     } elsif ($award eq 'ERROR') {
