@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # The LON-CAPA Homework handler
 #
-# $Id: lonhomework.pm,v 1.289 2008/01/15 03:09:09 raeburn Exp $
+# $Id: lonhomework.pm,v 1.293 2008/05/19 21:07:33 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -712,10 +712,6 @@ sub handle_save_or_undo {
 	my $fs=Apache::File->new(">$filebak");
 	if (defined($fs)) {
 	    print $fs $$problem;
-	    &Apache::lonxml::info("<b>".&mt("Making Backup to [_1]",
-					    '<span class="LC_filename">'.
-					    $filebak.'</span>').
-				  "</b>");
 	} else {
 	    &Apache::lonxml::info("<span class=\"LC_error\">".
 				  &mt("Unable to make backup [_1]",
@@ -726,9 +722,6 @@ sub handle_save_or_undo {
 	my $fh=Apache::File->new(">$file");
 	if (defined($fh)) {
 	    print $fh $$result;
-	    &Apache::lonxml::info("<b>".&mt("Saving Modifications to [_1]",
-					   '<span class="LC_filename">'.
-					    $file.'</span>' )."</b>");
 	} else {
 	    &Apache::lonxml::info('<span class="LC_error">'.
 				  &mt("Unable to write to [_1]",
@@ -1086,8 +1079,10 @@ sub get_template_list {
     }
     my @files = glob($Apache::lonnet::perlvar{'lonIncludes'}.
 		     '/templates/*.'.$glob_extension);
-    @files = map {[$_,&mt(&Apache::lonnet::metadata($_, 'title'))]} (@files);
-    @files = sort {$a->[1] cmp $b->[1]} (@files);
+    @files = map {[$_,&mt(&Apache::lonnet::metadata($_, 'title')),
+                      (&Apache::lonnet::metadata($_, 'category')?&mt(&Apache::lonnet::metadata($_, 'category')):&mt('Miscellaneous')),
+                      &mt(&Apache::lonnet::metadata($_, 'help'))]} (@files);
+    @files = sort {$a->[2].$a->[1] cmp $b->[2].$b->[1]} (@files);
     my ($midpoint,$seconddiv,$numfiles);
     $numfiles = 0;
     foreach my $file (@files) {
@@ -1102,16 +1097,28 @@ sub get_template_list {
         }
     }
     my $count = 0;
+    my $currentcategory='';
     foreach my $file (@files) {
 	next if ($file->[1] !~ /\S/);
+        if ($file->[2] ne $currentcategory) {
+           $currentcategory=$file->[2];
+           if ((!$seconddiv) && ($count >= $midpoint)) {
+               $result .= '</div>'."\n".'<div class="LC_left_float">'."\n";
+               $seconddiv = 1;
+           }
+           $result.='<h3>'.$currentcategory.'</h3>';
+           $count++;
+        }
 	$result .=
 	    '<label><input type="radio" name="template" value="'.$file->[0].'" />'.
-	    $file->[1].'</label><br />'."\n";
-        $count ++;
-        if ((!$seconddiv) && ($count >= $midpoint)) {
-            $result .= '</div>'."\n".'<div class="LC_left_float">'."\n";
-            $seconddiv = 1;
+	    $file->[1].'</label>';
+        if ($file->[3]) {
+           $result.=&Apache::loncommon::help_open_topic($file->[3]);
         }
+        my $filename=$file->[0];
+        $filename=~s/^\/home\/httpd\/html//;
+        $result.=' <font size="-2"><a href="'.$filename.'" target="sample">'.&mt('Example').'</font></a><br />'."\n";
+        $count ++;
     }
     if ($numfiles > 0) {
         $result .= '</div>'."\n".'<div class="LC_clear_float_footer"></div>'."\n";
@@ -1183,8 +1190,8 @@ sub update_construct_style {
 	    &Apache::lonnet::delenv('construct\\.style');
 	} elsif ($env{'form.style_file'} 
 	    && $env{'construct.style'} ne $env{'form.style_file'}) {
-	    &Apache::lonnet::appenv('construct.style' => 
-				        $env{'form.style_file'});
+	    &Apache::lonnet::appenv({'construct.style' => 
+				        $env{'form.style_file'}});
 	}
     }
 }
