@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Full matching style response
 #
-# $Id: matchresponse.pm,v 1.67 2007/10/15 09:47:29 foxr Exp $
+# $Id: matchresponse.pm,v 1.69 2008/02/01 22:05:45 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -70,8 +70,9 @@ sub start_matchresponse {
 	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
     } elsif ($target eq 'analyze') {
 	my $part_id="$Apache::inputtags::part.$id";
+        $Apache::lonhomework::analyze{"$part_id.type"} = 'matchresponse';
 	push (@{ $Apache::lonhomework::analyze{"parts"} },$part_id);
-	push (@{ $Apache::lonhomework::anaylze{"$part_id.bubble_lines"} },
+	push (@{ $Apache::lonhomework::analyze{"$part_id.bubble_lines"} },
 	      1);
     }
     return $result;
@@ -262,7 +263,7 @@ sub end_foilgroup {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
     my $result;
     my $part        = $Apache::inputtags::part;
-    my $response_id = $Apache::inputtages::response[-1];
+    my $response_id = $Apache::inputtags::response[-1];
 
     if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' || $target eq 'tex' || $target eq 'analyze') {
 	my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
