@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # random labelling tool
 #
-# $Id: randomlabel.pm,v 1.90 2007/10/17 22:11:55 albertel Exp $
+# $Id: randomlabel.pm,v 1.91 2008/03/12 02:46:53 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -178,7 +178,7 @@ sub end_randomlabel {
 	$count = $Apache::randomlabel::obj_cnt;
 	if( $count != 0) { $args{"cgi.$cgi_id.OBJCOUNT"}=$count; }
 	$result.='<img src="/adm/randomlabel.png?token='.$cgi_id.'" /><br />'."\n";
-	&Apache::lonnet::appenv(%args);
+	&Apache::lonnet::appenv(\%args);
     } elsif ($target eq 'tex') {
 	$result='\end{picture}\\\\';
 	$result.= ' \vskip -'.$height_param.' mm }  \\\\ ';
