@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA 
 # definition of tags that give a structure to a document
 #
-# $Id: structuretags.pm,v 1.419 2008/02/13 16:21:44 bisitz Exp $
+# $Id: structuretags.pm,v 1.423 2008/06/11 13:52:35 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -312,24 +312,28 @@ sub remember_problem_state {
        <input type="hidden" name="problemstatus" value="'.$env{'form.problemstatus'}.'" />';
 }
 
-sub problem_edit_header {
-    return '<input type="hidden" name="submitted" value="edit" />'.
-	&Apache::structuretags::remember_problem_state().'
-<div class="LC_edit_problem_header">
-<div class="LC_edit_problem_header_title">
-'.&mt('Problem Editing').&Apache::loncommon::help_open_menu('Problem Editing','Problem_Editor_XML_Index',5,'Authoring').'
-</div>
+sub problem_edit_buttons {
+   return  '
 <div class="LC_edit_problem_discards">
-       <input type="hidden" name="problemmode" value="edit" />
        <input type="button" name="submitmode" accesskey="d" value="'.&mt('Discard Edits and View').'" '.
-       ' onclick="javscript:setmode(this.form,'."'discard.'".')"  />
+       ' onclick="javscript:setmode(this.form,'."'discard'".')"  />
        <input '.&Apache::edit::submit_ask_anyway('setmode(this.form,'."'editxml'".')').' type="button" name="submitmode" accesskey="x" value="'.&mt('EditXML').'" />
        <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
 </div>
 <div class="LC_edit_problem_saves">
        <input type="submit" name="submitbutton" accesskey="s" value="'.&mt('Save and Edit').'" />
        <input type="submit" name="submitbutton" accesskey="v" value="'.&mt('Save and View').'" />
-  </div>
+</div>';
+}
+
+sub problem_edit_header {
+    return '<input type="hidden" name="submitted" value="edit" /><input type="hidden" name="problemmode" value="edit" />'.
+	&Apache::structuretags::remember_problem_state().'
+<div class="LC_edit_problem_header">
+<div class="LC_edit_problem_header_title">
+'.&mt('Problem Editing').&Apache::loncommon::help_open_menu('Problem Editing','Problem_Editor_XML_Index',5,'Authoring').'
+</div>'.
+&problem_edit_buttons().'
 <hr class="LC_edit_problem_divide" />
 '.&Apache::lonxml::message_location().'
 </div>
@@ -340,17 +344,8 @@ sub problem_edit_header {
 sub problem_edit_footer {
     return '</td></tr></table><br />
 <div class="LC_edit_problem_footer">
-  <hr class="LC_edit_problem_divide" />
-  <div class="LC_edit_problem_discards">
-     <input type="button" name="submitmode" accesskey="d" value="'.&mt('Discard Edits and View').'" '.
-     'onclick=javascript:setmode(this.form,'."'discard'".')" />
-     <input '.&Apache::edit::submit_ask_anyway('setmode(this.form,'."'editxml'".')').' type="button" name="submitmode" accesskey="x" value="'.&mt('EditXML').'" />
-     <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
-  </div>
-  <div class="LC_edit_problem_saves">
-    <input type="submit" name="submitbutton" value="'.&mt('Save and Edit').'" />
-    <input type="submit" name="submitbutton" value="'.&mt('Save and View').'" />
-  </div>
+  <hr class="LC_edit_problem_divide" />'.
+&problem_edit_buttons().'
   <hr class="LC_edit_problem_divide" />
 </div>
 '.
@@ -451,7 +446,6 @@ $show_all
 $show_all
 ";
     }
-
     $result.='
        <span class="LC_nobreak">
        '.&mt('Apply style file: ').'
@@ -459,6 +453,9 @@ $show_all
          <a href="javascript:openbrowser(\'lonhomework\',\'style_file\',\'sty\')">'.&mt('Select').'</a>
        </span>
      </div>
+     <div class="LC_edit_problem_header_row1">'.
+       &Apache::lonxml::renderingoptions().'
+     </div>
      <input type="submit" name="changeproblemmode" value="'.&mt("Change View").'" />
      <input type="submit" name="clear_style_file" accesskey="d" value="'.&mt('Show Default View').'" />
      <input type="submit" name="resetdata" accesskey="r" value="'.&mt('Reset Submissions').'" />
