@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler for displaying the course catalog interface
 #
-# $Id: coursecatalog.pm,v 1.26 2008/02/15 02:30:14 raeburn Exp $
+# $Id: coursecatalog.pm,v 1.37 2008/07/08 09:23:38 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -54,7 +54,8 @@ sub handler {
     &Apache::lonacc::get_posted_cgi($r);
     &Apache::lonlocal::get_language_handle($r);
     &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['sortby']);
-    my $codedom = &Apache::lonnet::default_login_domain(); 
+
+    my $codedom = &Apache::lonnet::default_login_domain();
 
     if (($env{'user.domain'} ne '') && ($env{'user.domain'} ne 'public')) { 
         $codedom = $env{'user.domain'};
@@ -63,23 +64,93 @@ sub handler {
         }
     }
     my $formname = 'coursecatalog';
-    my $domdesc = &Apache::lonnet::domain($codedom,'description');
-
-    &Apache::lonhtmlcommon::clear_breadcrumbs();
-    if ($env{'form.coursenum'} ne '' && &user_is_known()) {
-        &course_details($r,$codedom,$formname,$domdesc);
-    } else {
-        my $numtitles = &course_selector($r,$codedom,$formname,$domdesc);
-        if ($env{'form.state'} eq 'listing') {
-            $r->print(&print_course_listing($codedom,$numtitles).'<br />');
+    if ($env{'form.showdom'} ne '') {
+        if (&Apache::lonnet::domain($env{'form.showdom'}) ne '') {
+            $codedom = $env{'form.showdom'};
         }
     }
-    $r->print(&Apache::loncommon::end_page());
+    my $domdesc = &Apache::lonnet::domain($codedom,'description');
+    &Apache::lonhtmlcommon::clear_breadcrumbs();
+
+    my %domconfig =
+        &Apache::lonnet::get_dom('configuration',['coursecategories'],$codedom);
+    my (@cats,@trails,%allitems,%idx,@jsarray,%subcathash,$cathash);
+    if (ref($domconfig{'coursecategories'}) eq 'HASH') {
+        $cathash = $domconfig{'coursecategories'}{'cats'};
+    } else {
+        $cathash = {};
+    }
+    my $subcats;
+    if ($env{'form.withsubcats'}) {
+        $subcats = \%subcathash;
+    }
+    &Apache::loncommon::extract_categories($cathash,\@cats,\@trails,\%allitems,
+                                           \%idx,\@jsarray,$subcats);
+    if ($env{'form.coursenum'} ne '' && &user_is_known()) {
+        &course_details($r,$codedom,$formname,$domdesc,\@trails,\%allitems);
+    } else {
+        my ($catlinks,$has_subcats) = &category_breadcrumbs($codedom,@cats);
+        my $catjs = <<"ENDSCRIPT";
+
+function setCatDepth(depth) {
+    document.coursecats.catalog_maxdepth.value = depth;
+    if (depth == '') {
+        document.coursecats.currcat_0.value = '';
+    }
+    document.coursecats.submit();
+    return;
+}
+
+function changeSort(caller) {
+    document.$formname.sortby.value = caller;
+    document.$formname.submit();
+}
+function setCourseId(caller) {
+    document.$formname.coursenum.value = caller;
+    document.$formname.submit();
+}
+
+ENDSCRIPT
+        my $numtitles;
+        if ($env{'form.currcat_0'} eq 'instcode::0') {
+            $numtitles = &instcode_course_selector($r,$codedom,$formname,$domdesc,
+                                                   $catlinks,$catjs);
+            if ($env{'form.state'} eq 'listing') {
+                $r->print(&print_course_listing($codedom,$numtitles));
+            }
+        } else {
+            my (%add_entries);
+            $catjs = '<script type="text/javascript">'."\n".$catjs."\n".'</script>';
+            &cat_header($r,$codedom,$catjs,\%add_entries,$catlinks);
+            if ($env{'form.currcat_0'} ne '') {
+                $r->print('<form name="'.$formname.
+                          '" method="post" action="/adm/coursecatalog">'.
+                          &additional_filters($codedom,$has_subcats)."\n");
+                my ($currdepth,$deeper) = &get_depth_values();
+                $r->print('<input type="hidden" name="catalog_maxdepth" value="'.
+                          $deeper.'" />'."\n");
+                for (my $i=0; $i<$deeper; $i++) {
+                    $r->print('<input type="hidden" name="currcat_'.$i.'" value="'.$env{'form.currcat_'.$i}.'" />'."\n");
+                }
+                $r->print('<input type="hidden" name="coursenum" value="" />'."\n".
+                          '<input type="hidden" name="sortby" value="" />'."\n".
+                          '<input type="hidden" name="state" value="listing" />'."\n".
+                          '<input type="hidden" name="showdom" value="'.
+                          $env{'form.showdom'}.'" />'.
+                          '<input type="submit" name="catalogfilter" value="'.
+                          &mt('Display courses').'" /></form><br /><br />');
+            }
+            if ($env{'form.state'} eq 'listing') {
+                $r->print(&print_course_listing($codedom,undef,\@trails,\%allitems,$subcats));
+            }
+        }
+    }
+    $r->print('<br />'.&Apache::loncommon::end_page());
     return OK;
 }
 
 sub course_details {
-    my ($r,$codedom,$formname,$domdesc) = @_;
+    my ($r,$codedom,$formname,$domdesc,$trails,$allitems) = @_;
     my $output;
     my %add_entries = (topmargin    => "0",
                        marginheight => "0",);
@@ -100,8 +171,9 @@ sub course_details {
              {text=>"Course details"});
     $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Details'));
     $r->print('<br />'.&mt('Detailed course information:').'<br /><br />'.
-              '<form name="coursecatalog" method="post">'.
-              &print_course_listing($codedom).'<br /><br />');
+              '<form name="'.$formname.'" method="post">'.
+              &print_course_listing($codedom,undef,$trails,$allitems).
+              '<br /><br />');
     $r->print('<a href = "javascript:document.coursecatalog.submit()">'.
               &mt('Back to course listing').'</a>'.
               '<input type="hidden" name="sortby" value="'.
@@ -109,8 +181,8 @@ sub course_details {
               '<input type="hidden" name="state" value="listing" /></form>');
 }
 
-sub course_selector {
-    my ($r,$codedom,$formname,$domdesc) = @_;
+sub instcode_course_selector {
+    my ($r,$codedom,$formname,$domdesc,$catlinks,$catjs) = @_;
     my %coursecodes = ();
     my %codes = ();
     my @codetitles = ();
@@ -128,6 +200,9 @@ sub course_selector {
     my $totcodes = 0;
     my $jscript = '';
     my ($numtitles,$lasttitle);
+    my %add_entries = (topmargin    => "0",
+                       marginheight => "0",);
+    my $js;
     $totcodes = &Apache::courseclassifier::retrieve_instcodes(\%coursecodes,$codedom);
     if ($totcodes > 0) {
         $format_reply = &Apache::lonnet::auto_instcode_format($caller,$codedom,\%coursecodes,\%codes,\@codetitles,\%cat_titles,\%cat_order);
@@ -183,52 +258,21 @@ function setElements() {
                     $jscript .= '}';
                 }
             }
-            if ($env{'form.state'} eq 'listing') {
-                $jscript .= qq|
-function changeSort(caller) {
-    document.$formname.sortby.value = caller;
-    document.$formname.submit();
-}
-function setCourseId(caller) {
-   document.$formname.coursenum.value = caller;
-   document.$formname.submit(); 
-}\n|;
-            }
         }
-        my $js = '<script type"text/javascript">'."\n$jscript\n".
-                 '</script>';
-        my %add_entries = (topmargin    => "0",
-                           marginheight => "0",);
+        $js = '<script type"text/javascript">'."\n$jscript\n$catjs\n".
+              '</script>';
         if (($env{'form.state'} eq 'listing') && ($numtitles > 0)) {
             $add_entries{'onLoad'} = 'setElements()';
         }
-        my $start_page =
-            &Apache::loncommon::start_page('Course Catalog',$js,
-                                           {
-                                             'add_entries' => \%add_entries,
-                                             'no_inline_link'   => 1,});
-        $r->print($start_page);
-        if ($env{'form.state'} eq 'listing') {
-            if ($numtitles > 0) {
-                &Apache::lonhtmlcommon::add_breadcrumb
-                ({href=>"/adm/coursecatalog",
-                  text=>"Select courses"},
-                 {text=>"Course listing"});
-            } else {
-                &Apache::lonhtmlcommon::add_breadcrumb
-                ({text=>"Course listing"});
-            }
-            $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Listing'));
-        } else {
-            &Apache::lonhtmlcommon::add_breadcrumb
-            ({href=>"/adm/coursecatalog",
-              text=>"Select courses"});
-            $r->print(&Apache::lonhtmlcommon::breadcrumbs('Select courses'));
-        }
-        $r->print('<h3>'.&mt('Display information about official [_1] classes for which LON-CAPA courses have been created:',$domdesc).'</h3>');
-        $r->print('<form name="coursecatalog" method="post">');
+        &cat_header($r,$codedom,$js,\%add_entries,$catlinks,$numtitles);
+        my $cat_maxdepth = $env{'form.catalog_maxdepth'};
+        $r->print('<form name="'.$formname.'" method="post" action="/adm/coursecatalog">'.
+                  '<input type="hidden" name="catalog_maxdepth" value="'.$cat_maxdepth.'" />'."\n".
+                  '<input type="hidden" name="showdom" value="'.$env{'form.showdom'}.'" />'."\n".
+                  '<input type="hidden" name="currcat_0" value="instcode::0" />'.
+                  &additional_filters($codedom));
         if ($numtitles > 0) {
-            $r->print(&mt('<b>Choose which course(s) to list.</b><br />'));
+            $r->print('<b>'.&mt('Choose which course(s) to list.').'</b><br />');
             $r->print('<table><tr>');
             for (my $k=0; $k<$lasttitle-1; $k++) {
                 my (@items,@unsorted);
@@ -259,40 +303,227 @@ function setCourseId(caller) {
                       '<select name="'.$codetitles[$lasttitle-1].'">'."\n".
                       '<option value="0">All'."\n".
                       '</option>'."\n".'</select>'."\n".
-                 '</td>'
-                );
-            if (&user_is_dc($codedom)) {
-                my $showdetails_status;
-                if ($env{'form.showdetails'}) {
-                    $showdetails_status = 'checked="checked" ';
-                } 
-                $r->print('<td></td><td><input type="checkbox" name="showdetails" value="1" '.$showdetails_status.'/>'.&mt('Show full details for each course (DC only)').'</td>');
-            }
-            $r->print('</tr></table>');
+                 '</td></tr></table>'."\n");
             if ($numtitles > 4) {
                 $r->print('<br /><br />'.$codetitles[$numtitles-1].'<br />'."\n".
                 '<input type="text" name="'.$codetitles[$numtitles-1].'" /><br />'."\n");
             }
             $r->print('<br />');
         }
-        $r->print('<input type="hidden" name="coursenum" value="" /><input type="hidden" name="sortby" value="" /><input type="hidden" name="state" value="listing" />');
-        if ($numtitles > 0) {
-            $r->print('<input type="submit" name="catalogfilter" value="'.&mt('Display courses').'" />');
-        }
-        $r->print('<input type="hidden" name="numtitles" value="'.
-                  $numtitles.'" /></form>');
-        if (($numtitles > 0) && ($env{'form.state'} eq 'listing')) {
-            $r->print('<br /><br />');
-        }
+        $r->print('<input type="hidden" name="coursenum" value="" />'."\n".
+                  '<input type="hidden" name="sortby" value="" />'."\n".
+                  '<input type="hidden" name="state" value="listing" />'."\n".
+                  '<input type="hidden" name="form.currcat_0" value="instcode::0" />'."\n".
+                  '<input type="submit" name="catalogfilter" value="'.
+                  &mt('Display courses').'" />'.
+                  '<input type="hidden" name="numtitles" value="'.$numtitles.
+                  '" /></form><br /><br />');
     } else {
-        $r->print(&Apache::loncommon::start_page('Course Catalog','',
-                  {
-                   'no_inline_link'   => 1,}));
-        $r->print('<br />'.&mt('No official courses to display for [_1].',$domdesc));
+        &cat_header($r,$codedom,$js,\%add_entries,$catlinks,$numtitles);
+        my $cat_maxdepth = $env{'form.catalog_maxdepth'};
+        $r->print('<form name="'.$formname.'" method="post" action="/adm/coursecatalog">'.
+                  '<input type="hidden" name="catalog_maxdepth" value="'.$cat_maxdepth.'" />'.
+                  '<input type="hidden" name="showdom" value="'.$env{'form.showdom'}.'" />'.
+                  '<input type="hidden" name="currcat_0" value="instcode::0" />');
+        $r->print('<br />'.&mt('No official courses to display for [_1].',$domdesc).'</form>');
     }
     return $numtitles;
 }
 
+sub cat_header {
+    my ($r,$codedom,$js,$add_entries,$catlinks,$numtitles) = @_;
+    my $start_page =
+        &Apache::loncommon::start_page('Course Catalog',$js,
+                                       {
+                                         'add_entries' => $add_entries,
+                                         'no_inline_link'   => 1,});
+    $r->print($start_page);
+    if ($env{'form.state'} eq 'listing') {
+        if ($numtitles > 0) {
+            &Apache::lonhtmlcommon::add_breadcrumb
+                ({href=>"/adm/coursecatalog",
+                  text=>"Select courses"},
+                 {text=>"Course listing"});
+        } else {
+            &Apache::lonhtmlcommon::add_breadcrumb
+            ({text=>"Course listing"});
+        }
+        $r->print(&Apache::lonhtmlcommon::breadcrumbs('Course Listing'));
+    } else {
+        &Apache::lonhtmlcommon::add_breadcrumb
+        ({href=>"/adm/coursecatalog",
+          text=>"Select courses"});
+        $r->print(&Apache::lonhtmlcommon::breadcrumbs('Select courses'));
+    }
+    $r->print('<form name="coursecatdom" method="post" action="/adm/coursecatalog">'.
+              '<table border="0"><tr><td><b>'.&mt('Domain:').'</b></td><td>'.
+              &Apache::loncommon::select_dom_form($codedom,'showdom','',1).
+	      '&nbsp;<input type="submit" name="godom" value="'.&mt('Change').'" /></td></tr></table></form>'.
+	      '<form name="coursecats" method="post" action="/adm/coursecatalog">'.
+              '<table border="0"><tr>'.$catlinks.'</tr></table></form>');
+    return;
+}
+
+sub category_breadcrumbs {
+    my ($dom,@cats) = @_;
+    my ($currdepth,$deeper) = &get_depth_values();
+    my $currcat_str = '<input type="hidden" name="catalog_maxdepth" value="'.$deeper.'" /><input type="hidden" name="showdom" value="'.$dom.'" />';
+    my $catlinks = '<td valign="top"><b>'.&mt('Catalog:').'</b></td><td><table><tr>';
+    my $has_subcats;
+    for (my $i=0; $i<$deeper; $i++) {
+        $currcat_str .= '<input type="hidden" name="currcat_'.$i.'" value="'.$env{'form.currcat_'.$i}.'" />';
+        my ($cattitle,$shallower);
+        if ($i == 0) {
+            $cattitle = &mt('Main Categories');
+        } else {
+            $shallower = $i-1;
+            my ($cat,$container,$depth) = map { &unescape($_); } split(/:/,$env{'form.currcat_'.$shallower});
+            $cattitle = $cat;
+        }
+        $catlinks .= '<td valign="top"><a href="javascript:setCatDepth('."'$shallower'".')">'.$cattitle.'</a>-&gt;</td>';
+    }
+    if ($deeper == 0) {
+        $catlinks .= '<td>'.&mt('Main Categories').':&nbsp;';
+        if (ref($cats[0]) eq 'ARRAY') {
+            if ((@{$cats[0]} == 1) && ($cats[0][0] eq 'instcode')) {
+                $catlinks .= &mt('Official courses (with institutional codes)').
+                             '<input type="hidden" name="currcat_0" value="instcode::0" />';
+                $env{'form.currcat_0'} = 'instcode::0';
+            } else {
+                $has_subcats = 1;
+                $catlinks .= '<select name="currcat_0">'."\n";
+                if (@{$cats[0]} > 1) {
+                    my $selstr;
+                    if ($env{'form.currcat_0'} eq '') {
+                        $selstr = ' selected="selected" ';
+                    }
+                    $catlinks .= '<option value=""  selected="selected">'.&mt('Select').'</option>'."\n";
+                }
+                for (my $i=0; $i<@{$cats[0]}; $i++) {
+                    my $name = $cats[0][$i];
+                    my $item = &escape($name).'::0';
+                    $catlinks .= '<option value="'.$item.'">';
+                    if ($name eq 'instcode') {
+                        $catlinks .= &mt('Official courses (with institutional codes)');
+                    } else {
+                        $catlinks .= $name;
+                    }
+                    $catlinks .= '</option>'."\n";
+                }
+                $catlinks .= '</select>'."\n".
+                             '&nbsp;<input type="submit" name="gocats" value="'.&mt('Change').'" />';
+            }
+        } else {
+            $catlinks .= &mt('Official courses (with institutional codes)').
+                         '<input type="hidden" name="currcat_0" value="instcode::0" />';
+            $env{'form.currcat_0'} = 'instcode::0';
+        }
+    } else {
+        my ($cat,$container,$depth);
+        if ($env{'form.currcat_'.$currdepth} eq '') {
+            my $shallower = $currdepth - 1;
+            ($cat,$container,$depth) = map { &unescape($_); } split(/:/,$env{'form.currcat_'.$shallower});
+        } else {
+            ($cat,$container,$depth) = map { &unescape($_); } split(/:/,$env{'form.currcat_'.$currdepth});
+        }
+        my $deeper = $depth +1;
+        my $currcat = $cat;
+        if ($cat eq 'instcode') {
+            $currcat = &mt('Official courses (with institutional codes)');
+        }
+        $catlinks .= '<td>'.$currcat;
+        if (ref($cats[$deeper]{$cat}) eq 'ARRAY') {
+            $has_subcats = 1;
+            my $selstr;
+            $catlinks .= ':&nbsp;<select name="currcat_'.$deeper.'">'.
+                         '<option value="" selected="selected">'.
+                         &mt('Select').'</option>';
+            for (my $k=0; $k<@{$cats[$deeper]{$cat}}; $k++) {
+                my $name = $cats[$deeper]{$cat}[$k];
+                my $item = &escape($name).':'.&escape($cat).':'.$deeper;
+                $catlinks .= '<option value="'.$item.'">'.$name.'</option>'."\n";
+            }
+            $catlinks .= '</select>'."\n".
+                         '&nbsp;<input type="submit" name="gocats" value="'.&mt('Change').'" />';
+        }
+    }
+    $catlinks .= $currcat_str.'</td></tr></table></td>';
+    return ($catlinks,$has_subcats);
+}
+
+sub get_depth_values {
+    my $currdepth = 0;
+    my $deeper = 0;
+    if ($env{'form.catalog_maxdepth'} ne '') {
+        $currdepth = $env{'form.catalog_maxdepth'};
+        if ($env{'form.currcat_'.$currdepth} eq '') {
+            $deeper = $currdepth;
+        } else {
+            $deeper = $currdepth + 1;
+        }
+    }
+    return ($currdepth,$deeper);
+}
+
+sub additional_filters {
+    my ($codedom,$has_subcats) = @_;
+    my $output = '<table>';
+    if (($env{'form.currcat_0'} ne 'instcode::0') && 
+        ($env{'form.currcat_0'} ne '') && ($has_subcats)) {
+        my $include_subcat_status;
+        if ($env{'form.withsubcats'}) {
+            $include_subcat_status = 'checked="checked" ';
+        }
+        my $counter = $env{'form.catalog_maxdepth'};
+        if ($counter > 0) {
+            if ($env{'form.state'} eq 'listing') {
+                $counter --;
+            } elsif ($env{'form.currcat_'.$counter} eq '') {
+                $counter --;
+            }
+        }
+        my ($catname) = split(/:/,$env{'form.currcat_'.$counter});
+        if ($catname ne '') {
+            $output .= '<tr><td><label>'.
+                       '<input type="checkbox" name="withsubcats" value="1" '.
+                       $include_subcat_status.'/>'.
+                       &mt('Include subcategories within "[_1]"',$catname).
+                       '</label></td></tr>';
+        }
+    }
+    my $show_selfenroll_status;
+    if ($env{'form.showselfenroll'}) {
+        $show_selfenroll_status = 'checked="checked" ';
+    }
+    $output .= '<tr><td>'.
+               '<label><input type="checkbox" name="showselfenroll" value="1" '.
+               $show_selfenroll_status.'/>'.
+               &mt('Only show courses which allow self-enrollment').
+               '</label></td></tr>';
+    if (&user_is_dc($codedom)) {
+        my $showdetails_status;
+        if ($env{'form.showdetails'}) {
+            $showdetails_status = 'checked="checked" ';
+        }
+        my $showhidden_status;
+        if ($env{'form.showhidden'}) {
+             $showhidden_status = 'checked="checked" ';
+        }
+        my $dc_title = &Apache::lonnet::plaintext('dc');
+        $output .= '<tr><td>'."\n".
+                   '<label><input type="checkbox" name="showdetails" value="1" '.
+                   $showdetails_status.'/>'.
+                   &mt('Show full details for each course ([_1] only)',$dc_title).
+                   '</label>'."\n".'</td></tr><tr><td>'.
+                   '<label><input type="checkbox" name="showhidden" value="1" '.
+                   $showhidden_status.'/>'.
+                   &mt('Include courses set to be hidden from catalog ([_1] only)',$dc_title).
+                   '</label>'."\n".'</td></tr>';
+    }
+    $output .= '</table><br />';
+    return $output;
+}
+
 sub user_is_dc {
     my ($codedom) = @_;
     if (exists($env{'user.role.dc./'.$codedom.'/'})) {
@@ -389,7 +620,7 @@ sub build_javascript {
     return $output;
 }
 
-sub search_courselist {
+sub search_official_courselist {
     my ($domain,$numtitles) = @_;
     my $instcode;
     if (defined($numtitles) && $numtitles == 0) {
@@ -413,14 +644,58 @@ sub search_courselist {
             $instcode = '.';
         }
     }
-    my %courses = &Apache::lonnet::courseiddump($domain,'.',1,$instcode,'.','.',
-                                                undef,undef,'Course',1);
+    my $showhidden;
+    if (&user_is_dc($domain)) {
+        $showhidden = $env{'form.showhidden'};
+    }
+    my %courses = 
+        &Apache::lonnet::courseiddump($domain,'.',1,$instcode,'.','.',undef,undef,
+                                      'Course',1,$env{'form.showselfenroll'},undef,
+                                      $showhidden,'coursecatalog');
     return %courses;
 }
 
+sub search_courselist {
+    my ($domain,$subcats) = @_;
+    my $cat_maxdepth = $env{'form.catalog_maxdepth'};
+    my $filter = $env{'form.currcat_'.$cat_maxdepth};
+    if (($filter eq '') && ($cat_maxdepth > 0)) {
+        my $shallower = $cat_maxdepth - 1;
+        $filter = $env{'form.currcat_'.$shallower};
+    }
+    my %courses;
+    my $filterstr;
+    if ($filter ne '') {
+        if ($env{'form.withsubcats'}) {
+            if (ref($subcats) eq 'HASH') {
+                if (ref($subcats->{$filter}) eq 'ARRAY') {
+                    $filterstr = join('&',@{$subcats->{$filter}});
+                    if ($filterstr ne '') {
+                        $filterstr = $filter.'&'.$filterstr;
+                    }
+                } else {
+                    $filterstr = $filter;
+                }
+            } else {
+                $filterstr = $filter;
+            }  
+        } else {
+            $filterstr = $filter; 
+        }
+        my $showhidden;
+        if (&user_is_dc($domain)) {
+            $showhidden = $env{'form.showhidden'};
+        }
+        %courses = 
+            &Apache::lonnet::courseiddump($domain,'.',1,'.','.','.',undef,undef,
+                                          '.',1,$env{'form.showselfenroll'},
+                                          $filterstr,$showhidden,'coursecatalog');
+    }
+    return %courses;
+}
 
 sub print_course_listing {
-    my ($domain,$numtitles) = @_;
+    my ($domain,$numtitles,$trails,$allitems,$subcats) = @_;
     my $output;
     my %courses;
     my $knownuser = &user_is_known();
@@ -433,37 +708,56 @@ sub print_course_listing {
     if ($env{'form.coursenum'} ne '') {
         %courses = &Apache::lonnet::courseiddump($domain,'.',1,'.','.',
                                                  $env{'form.coursenum'},
-                                                 undef,undef,'Course',1);
+                                                 undef,undef,'.',1);
         if (keys(%courses) == 0) {
             $output .= &mt('The courseID provided does not match a course in this domain.');
             return $output;
         }
     } else {
-        %courses = &search_courselist($domain,$numtitles);
+        if ($env{'form.currcat_0'} eq 'instcode::0') {
+            %courses = &search_official_courselist($domain,$numtitles);
+        } else {
+            %courses = &search_courselist($domain,$subcats);
+        }
         if (keys(%courses) == 0) {
             $output = &mt('No courses match the criteria you selected.');
             return $output;
         }
-        if ($knownuser && !$env{'form.showdetails'}) {
-            $output = &mt('<b>Note for students:</b> If you are officially enrolled in a course but the course is not listed in your LON-CAPA courses, click the "Show more details" link for the specific course and check the default access dates and/or automated enrollment settings.<br /><br />');
+        if (($knownuser) && (!$env{'form.showdetails'}) && (!&user_is_dc($domain))) {
+            $output = '<b>'.&mt('Note for students:').'</b> '
+                     .&mt('If you are officially enrolled in a course but the course is not listed in your LON-CAPA courses, click the "Show more details" link for the specific course and check the default access dates and/or automated enrollment settings.')
+                     .'<br /><br />';
         }
     }
-    $output .= &construct_data_table($knownuser,\%courses,$details);
-    $output .= &Apache::lonhtmlcommon::echo_form_input(['coursenum','state','catalogfilter','sortby','showdetails']);
+    my $now = time;
+    my %domconfig =
+        &Apache::lonnet::get_dom('configuration',['usercreation'],$domain);
+    $output .= &construct_data_table($knownuser,\%courses,$details,undef,$now,\%domconfig,$trails,$allitems);
+    if ($env{'form.coursenum'} ne '') {
+        $output .= &Apache::lonhtmlcommon::echo_form_input(['coursenum','state','catalogfilter','sortby','showdetails']);
+    }
     return $output;
 }
 
 sub construct_data_table {
-    my ($knownuser,$courses,$details,$usersections) = @_;
+    my ($knownuser,$courses,$details,$usersections,$now,$domconfig,$trails,
+        $allitems) = @_;
     my %sortname;
     if (($details eq '') || ($env{'form.showdetails'})) {
         $sortname{'Code'} = 'code';
+        $sortname{'Categories'} = 'cats';
         $sortname{'Title'} = 'title';
         $sortname{'Owner(s)'} = 'owner';
     }
     my $output = &Apache::loncommon::start_data_table().
                  &Apache::loncommon::start_data_table_header_row();
-    my @coltitles = ('Count','Code','Sections','Crosslisted','Title','Owner(s)');
+    my @coltitles = ('Count');
+    if ($env{'form.currcat_0'} eq 'instcode::0') {
+        push(@coltitles,'Code');
+    } else {
+        push(@coltitles,'Categories');
+    }
+    push(@coltitles,('Sections','Crosslisted','Title','Owner(s)'));
     if (ref($usersections) eq 'HASH') {
        $coltitles[1] = 'Your Section';
     }
@@ -485,9 +779,10 @@ sub construct_data_table {
               '<th>'.&mt('Student Counts').'</th>'.
               '<th>'.&mt('Auto-enrollment of <br />registered students').'</th>';
         } else {
-            $output .= '<th>&nbsp;</th>';
+            $output .= '<th>'.&mt('Details').'</th>';
         }
     }
+    $output .= '<th>'.&mt('Self-enroll (if permitted)').'</th>';
     &Apache::loncommon::end_data_table_header_row();
     my %courseinfo = &build_courseinfo_hash($courses,$knownuser,$details,
                                             $usersections);
@@ -495,6 +790,8 @@ sub construct_data_table {
     foreach my $course (sort(keys(%{$courses}))) {
         if ($env{'form.sortby'} eq 'code') {
             push(@{$Sortby{$courseinfo{$course}{'code'}}},$course);
+        } elsif ($env{'form.sortby'} eq 'cats') {
+            push(@{$Sortby{$courseinfo{$course}{'categories'}}},$course);
         } elsif ($env{'form.sortby'} eq 'owner') {
             push(@{$Sortby{$courseinfo{$course}{'ownerlastnames'}}},$course);
         } else {
@@ -507,7 +804,8 @@ sub construct_data_table {
         }
     }
     my @sorted_courses;
-    if (($env{'form.sortby'} eq 'code') || ($env{'form.sortby'} eq 'owner')) {
+    if (($env{'form.sortby'} eq 'code') || ($env{'form.sortby'} eq 'owner') ||
+        ($env{'form.sortby'} eq 'cats')) {
         @sorted_courses = sort(keys(%Sortby));
     } else {
         @sorted_courses = sort { lc($a) cmp lc($b) } (keys(%Sortby));
@@ -516,7 +814,8 @@ sub construct_data_table {
     foreach my $item (@sorted_courses) {
         foreach my $course (@{$Sortby{$item}}) {
             $output.=&Apache::loncommon::start_data_table_row(); 
-            $output.=&courseinfo_row($courseinfo{$course},$knownuser,$details,\$count);
+            $output.=&courseinfo_row($courseinfo{$course},$knownuser,$details,
+                                     \$count,$now,$course,$trails,$allitems);
             $output.=&Apache::loncommon::end_data_table_row();
         }
     }
@@ -537,12 +836,17 @@ sub build_courseinfo_hash {
         $cleandesc=~s/'/\\'/g;
         $cleandesc =~ s/^\s+//;
         my ($cdom,$cnum)=split(/\_/,$course);
-        my ($descr,$instcode,$singleowner,$ttype,@owners,%ownernames);
+        my ($descr,$instcode,$singleowner,$ttype,$selfenroll_types,
+            $selfenroll_start,$selfenroll_end,@owners,%ownernames,$categories);
         if (ref($courses->{$course}) eq 'HASH') {
             $descr = $courses->{$course}{'description'};
             $instcode =  $courses->{$course}{'inst_code'};
             $singleowner = $courses->{$course}{'owner'};
             $ttype =  $courses->{$course}{'type'};
+            $selfenroll_types = $courses->{$course}{'selfenroll_types'};
+            $selfenroll_start = $courses->{$course}{'selfenroll_start_date'};
+            $selfenroll_end = $courses->{$course}{'selfenroll_end_date'};
+            $categories = $courses->{$course}{'categories'};
             push(@owners,$singleowner);
             if (ref($courses->{$course}{'co-owners'}) eq 'ARRAY') {
                 foreach my $item (@{$courses->{$course}{'co-owners'}}) {
@@ -577,6 +881,10 @@ sub build_courseinfo_hash {
         $courseinfo{$course}{'ownerlastnames'} = join(', ',sort(@lastnames));
         $courseinfo{$course}{'title'} = $cleandesc;
         $courseinfo{$course}{'owner'} = $singleowner;
+        $courseinfo{$course}{'selfenroll_types'} = $selfenroll_types;
+        $courseinfo{$course}{'selfenroll_start'} = $selfenroll_start;
+        $courseinfo{$course}{'selfenroll_end'} = $selfenroll_end;
+        $courseinfo{$course}{'categories'} = $categories;
 
         my %coursehash = &Apache::lonnet::dump('environment',$cdom,$cnum);
         my @classids;
@@ -612,7 +920,7 @@ sub build_courseinfo_hash {
             if ( defined($coursehash{'default_enrollment_end_date'}) ) {
                 $endaccess = &Apache::lonlocal::locallocaltime($coursehash{'default_enrollment_end_date'});
                 if ($coursehash{'default_enrollment_end_date'} == 0) {
-                    $endaccess = "No ending date";
+                    $endaccess = &mt('No ending date');
                 }
             }
             if ($startaccess) {
@@ -621,6 +929,33 @@ sub build_courseinfo_hash {
             if ($endaccess) {
                 $accessdates .= &mt('<i>To:</i> ').$endaccess.'<br />';
             }
+            if (($selfenroll_types ne '') && 
+                ($selfenroll_end > 0 && $selfenroll_end > $now)) {
+                my ($selfenroll_start_access,$selfenroll_end_access);
+                if (($coursehash{'default_enrollment_start_date'} ne 
+                     $coursehash{'internal.selfenroll_start_access'}) ||
+                   ($coursehash{'default_enrollment_end_date'} ne 
+                    $coursehash{'internal.selfenroll_end_access'})) {
+                    if ( defined($coursehash{'internal.selfenroll_start_access'}) ) {
+                        $selfenroll_start_access = &Apache::lonlocal::locallocaltime($coursehash{'internal.selfenroll_start_access'});
+                    }
+                    if ( defined($coursehash{'default_enrollment_end_date'}) ) {
+                        $selfenroll_end_access = &Apache::lonlocal::locallocaltime($coursehash{'internal.selfenroll_end_access'});
+                        if ($coursehash{'internal.selfenroll_end_access'} == 0) {
+                            $selfenroll_end_access = &mt('No ending date');
+                        }
+                    }
+                    if ($selfenroll_start_access || $selfenroll_end_access) {
+                        $accessdates .= '<br/><br /><i>'.&mt('Self-enrollers:').'</i><br />';
+                        if ($selfenroll_start_access) {
+                            $accessdates .= &mt('<i>From:</i> ').$selfenroll_start_access.'<br />';
+                        }
+                        if ($selfenroll_end_access) {
+                            $accessdates .= &mt('<i>To:</i> ').$selfenroll_end_access.'<br />';
+                        }
+                    }
+                }
+            }
             $courseinfo{$course}{'access'} = $accessdates;
         }
         if ($xlist_items eq '') {
@@ -660,9 +995,9 @@ sub count_students {
 }
 
 sub courseinfo_row {
-    my ($info,$knownuser,$details,$countref) = @_;
+    my ($info,$knownuser,$details,$countref,$now,$course,$trails,$allitems) = @_;
     my ($cdom,$cnum,$title,$ownerlast,$code,$owner,$seclist,$xlist_items,
-        $accessdates,$showsyllabus,$counts,$autoenrollment,$output);
+        $accessdates,$showsyllabus,$counts,$autoenrollment,$output,$categories);
     if (ref($info) eq 'HASH') {
         $cdom = $info->{'cdom'};
         $cnum = $info->{'cnum'};
@@ -676,14 +1011,30 @@ sub courseinfo_row {
         $counts = $info->{'counts'};
         $autoenrollment = $info->{'autoenrollment'};
         $showsyllabus = $info->{'showsyllabus'};
+        $categories = $info->{'categories'};
     } else {
         $output = '<td colspan="8">'.&mt('No information available for [_1].',
                                          $code).'</td>';
         return $output;
     }
-    $output .= '<td>'.$$countref.'</td>'.
-               '<td>'.$code.'</td>'.
-               '<td>'.$seclist.'</td>'.
+    $output .= '<td>'.$$countref.'</td>';
+    if ($env{'form.currcat_0'} eq 'instcode::0') {
+        $output .= '<td>'.$code.'</td>';
+    } else {
+        my ($categorylist,@cats);
+        if ($categories ne '') {
+            @cats = split('&',$categories);
+        }
+        if ((ref($trails) eq 'ARRAY') && (ref($allitems) eq 'HASH')) {
+            my @categories = map { $trails->[$allitems->{$_}]; } @cats;
+            $categorylist = join('<br />',@categories);
+        }
+        if ($categorylist eq '') {
+            $categorylist = '&nbsp;';
+        }
+        $output .= '<td>'.$categorylist.'</td>';
+    }
+    $output .= '<td>'.$seclist.'</td>'.
                '<td>'.$xlist_items.'</td>'.
                '<td>'.$title.'&nbsp;<font size="-2">';
     if ($showsyllabus) {
@@ -703,6 +1054,22 @@ sub courseinfo_row {
             $output .= "<td><a href=\"javascript:setCourseId('$cnum')\">".&mt('Show more details').'</a></td>';
         }
     }
+    my $selfenroll;
+    if ($info->{'selfenroll_types'}) {
+        my $showstart = &Apache::lonlocal::locallocaltime($info->{'selfenroll_start'});
+        my $showend = &Apache::lonlocal::locallocaltime($info->{'selfenroll_end'});
+        if (($info->{'selfenroll_end'} > 0) && ($info->{'selfenroll_end'} > $now)) {
+            if (($info->{'selfenroll_start'} > 0) && ($info->{'selfenroll_start'} > $now)) {
+                $output .= '<td>'.&mt('Starts: [_1]','<span class="LC_cusr_emph">'.$showstart.'</span>').'<br />'.&mt('Ends: [_1]','<span class="LC_cusr_emph">'.$showend.'</span>').'</td>';
+            } else { 
+                $output .= '<td><a href="/adm/selfenroll?cid='.$course.'">'.&mt('Enroll in course').'</a></td>';
+            }
+            $selfenroll = 1;
+        }
+    }
+    if (!$selfenroll) {
+        $output .= '<td>&nbsp;</td>';
+    }
     $$countref ++;
     return $output;
 }
