@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Announce
 #
-# $Id: lonannounce.pm,v 1.69 2007/05/15 18:55:22 raeburn Exp $
+# $Id: lonannounce.pm,v 1.70 2008/05/18 19:45:46 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -523,7 +523,7 @@ SERVERANNOUNCE
 	    $r->print ('<br /><label><input type="checkbox" name="postto_'.$host.'" /> '.
 		       $host.' <tt>'.$hostname{$host}.'</tt> '.
 		       '</label><a href="http://'.$hostname{$host}.
-		       '/announcement.txt" target="annowin">current</a>');
+		       '/announcement.txt?time='.time.'" target="annowin">current</a>');
 	}
     }
     $r->print(
