@@ -153,11 +153,11 @@ SCRIPT_TWO
  <table id="LC_helpmenu_links">
    <tr>
 	<td><span class="LC_nobreak"><a href="/adm/help/$general_help.hlp" target="bodyframe">
-		  <img src="$location/help/gif/smallHelp.gif" border="0" alt="(General help)" valign="middle" />&nbsp;General Help</a>&nbsp;</span></td>
+		  <img src="'.$location.'/help/help.png" border="0" alt="(General help)" valign="middle" />&nbsp;General Help</a>&nbsp;</span></td>
 END
     if ($component_url) {
 	$r->print("<td><span class=\"LC_nobreak\"><a href=\"$component_url\" target=\"bodyframe\">".
-		  '<img src="'.$location.'/help/gif/smallHelp.gif" border="0" alt="(Topic help)" valign="middle" />&nbsp;'.$topic.'</a>&nbsp;</span></td>');
+		  '<img src="'.$location.'/help/help.png" border="0" alt="(Topic help)" valign="middle" />&nbsp;'.$topic.'</a>&nbsp;</span></td>');
     }
     if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
         $r->print('
