@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # a pile of common html routines
 #
-# $Id: lonhtmlcommon.pm,v 1.173 2008/03/06 22:30:17 raeburn Exp $
+# $Id: lonhtmlcommon.pm,v 1.175 2008/06/01 00:04:39 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -311,6 +311,7 @@ The method used to restrict user input will fail in the year 2400.
 sub date_setter {
     my ($formname,$dname,$currentvalue,$special,$includeempty,$state,
         $no_hh_mm_ss,$defhour,$defmin,$defsec,$nolink) = @_;
+    my $now = time;
     my $wasdefined=1;
     if (! defined($state) || $state ne 'disabled') {
         $state = '';
@@ -319,28 +320,25 @@ sub date_setter {
         $no_hh_mm_ss = 0;
     }
     if ($currentvalue eq 'now') {
-	$currentvalue=time;
+	$currentvalue = $now;
     }
     if ((!defined($currentvalue)) || ($currentvalue eq '')) {
 	$wasdefined=0;
 	if ($includeempty) {
 	    $currentvalue = 0;
 	} else {
-	    $currentvalue = time;
+	    $currentvalue = $now;
 	}
     }
     # other potentially useful values:     wkday,yrday,is_daylight_savings
+    my $tzname;
     my ($sec,$min,$hour,$mday,$month,$year)=('','',undef,'','','');
     if ($currentvalue) {
-	($sec,$min,$hour,$mday,$month,$year,undef,undef,undef) = 
-	    localtime($currentvalue);
-	$year += 1900;
+        ($tzname,$sec,$min,$hour,$mday,$month,$year) = &get_timedates($currentvalue); 
     }
     unless ($wasdefined) {
+        ($tzname,$sec,$min,$hour,$mday,$month,$year) = &get_timedates($now);
 	if (($defhour) || ($defmin) || ($defsec)) {
-	    ($sec,$min,$hour,$mday,$month,$year,undef,undef,undef) = 
-		localtime(time);
-	    $year += 1900;
 	    $sec=($defsec?$defsec:0);
 	    $min=($defmin?$defmin:0);
 	    $hour=($defhour?$defhour:0);
@@ -463,16 +461,19 @@ ENDJS
         $cal_link = qq{<a href="javascript:$dname\_opencalendar()">};
     }
     #
+    my $tzone = ' '.$tzname.' ';
     if ($no_hh_mm_ss) {
         $result .= &mt('[_1] [_2] [_3] ',
-                       $monthselector,$dayselector,$yearselector);
+                       $monthselector,$dayselector,$yearselector).
+                   $tzone;
         if (!$nolink) {
             $result .= &mt('[_1]Select Date[_2]',$cal_link,'</a>');
         }
     } else {
         $result .= &mt('[_1] [_2] [_3] [_4] [_5]m [_6]s ',
                       $monthselector,$dayselector,$yearselector,
-                      $hourselector,$minuteselector,$secondselector);
+                      $hourselector,$minuteselector,$secondselector).
+                   $tzone;
         if (!$nolink) {
             $result .= &mt('[_1]Select Date[_2]',$cal_link,'</a>');
         }
@@ -481,6 +482,22 @@ ENDJS
     return $result;
 }
 
+sub get_timedates {
+    my ($epoch) = @_;
+    my $dt = DateTime->from_epoch(epoch => $epoch)
+                     ->set_time_zone(&Apache::lonlocal::gettimezone());
+    my $tzname = $dt->time_zone_short_name();
+    my $sec = $dt->second;
+    my $min = $dt->minute;
+    my $hour = $dt->hour;
+    my $mday = $dt->day;
+    my $month = $dt->month;
+    if ($month) {
+        $month --;
+    }
+    my $year = $dt->year;
+    return ($tzname,$sec,$min,$hour,$mday,$month,$year);
+}
 
 sub build_url {
     my ($base, $fields)=@_;
@@ -562,20 +579,33 @@ sub get_date_from_form {
     if (defined($env{'form.'.$dname.'_month'})) {
         my $tmpmonth = $env{'form.'.$dname.'_month'};
         if (($tmpmonth =~ /^\d+$/) && ($tmpmonth > 0) && ($tmpmonth < 13)) {
-            $month = $tmpmonth - 1;
+            $month = $tmpmonth;
         }
     }
     if (defined($env{'form.'.$dname.'_year'})) {
         my $tmpyear = $env{'form.'.$dname.'_year'};
-        if (($tmpyear =~ /^\d+$/) && ($tmpyear > 1900)) {
-            $year = $tmpyear - 1900;
+        if (($tmpyear =~ /^\d+$/) && ($tmpyear >= 1970)) {
+            $year = $tmpyear;
         }
     }
-    if (($year<70) || ($year>137)) { return undef; }
+    if (($year<1970) || ($year>2037)) { return undef; }
     if (defined($sec) && defined($min)   && defined($hour) &&
-        defined($day) && defined($month) && defined($year) &&
-        eval('&timelocal($sec,$min,$hour,$day,$month,$year)')) {
-        return &timelocal($sec,$min,$hour,$day,$month,$year);
+        defined($day) && defined($month) && defined($year)) {
+        my $timezone = &Apache::lonlocal::gettimezone();
+        my $dt = DateTime->new( year   => $year,
+                                month  => $month,
+                                day    => $day,
+                                hour   => $hour,
+                                minute => $min,
+                                second => $sec,
+                                time_zone => $timezone,
+                              );
+        my $epoch_time  = $dt->epoch;
+        if ($epoch_time ne '') {
+            return $epoch_time;
+        } else {
+            return undef;
+        }
     } else {
         return undef;
     }
