@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Directory Indexer
 #
-# $Id: lonindexer.pm,v 1.179 2007/08/17 22:54:36 banghart Exp $
+# $Id: lonindexer.pm,v 1.180 2008/02/15 11:06:22 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -1014,7 +1014,7 @@ sub display_line {
 	&Apache::loncommon::inhibit_menu_check(\$quotable_filelink);
 	$r->print (" (<a href=\"javascript:openWindow('".$quotable_filelink.
 		   "?inhibitmenu=yes','metadatafile','500','550','no','yes','no');\" ".
-		   " target=\"_self\">metadata</a>) ");
+		   " target=\"_self\">".&mt('metadata')."</a>) ");
 # Close form to open/close sequence
 	if ($filelink=~/\.(page|sequence)$/) {
 	    $r->print('</form>');
