@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Metadata display handler
 #
-# $Id: lonmeta.pm,v 1.211 2008/01/24 10:05:13 bisitz Exp $
+# $Id: lonmeta.pm,v 1.212 2008/03/06 20:20:18 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -839,10 +839,14 @@ sub report_bombs {
 	    &Apache::lonmsg::clear_author_res_msg($uri);
 	}
         my $clear=&mt('Clear all Messages in Subdirectory');
-	$r->print(<<ENDCLEAR);
+        my $cancel=&mt('Back to Directory');
+        my $cancelurl=$uri;
+        $cancelurl=~s/^\Q$domain\E/\/priv/;
+        $r->print(<<ENDCLEAR);
 <form method="post">
 <input type="submit" name="clearbombs" value="$clear" />
-</form>
+<a href="$cancelurl">$cancel</a>
+</form><hr />
 ENDCLEAR
         my %brokenurls = 
             &Apache::lonmsg::all_url_author_res_msg($author,$domain);
