@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Navigate Maps Handler
 #
-# $Id: lonnavdisplay.pm,v 1.2 2007/04/20 01:54:29 albertel Exp $
+# $Id: lonnavdisplay.pm,v 1.4 2008/04/11 13:17:09 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -71,7 +71,7 @@ sub real_handler {
 				      "Select Action");
     if ($ENV{QUERY_STRING} eq 'collapseExternal') {
 	&Apache::lonnet::put('environment',{'remotenavmap' => 'off'});
-	&Apache::lonnet::appenv('environment.remotenavmap' => 'off');
+	&Apache::lonnet::appenv({'environment.remotenavmap' => 'off'});
 	my $menu=&Apache::lonmenu::reopenmenu();
 	my $navstatus=&Apache::lonmenu::get_nav_status();
 	if ($menu) {
@@ -110,7 +110,7 @@ ENDSUBM
     }
     if ($ENV{QUERY_STRING} =~ /^launchExternal/) {
 	&Apache::lonnet::put('environment',{'remotenavmap' => 'on'});
-	&Apache::lonnet::appenv('environment.remotenavmap' => 'on');
+	&Apache::lonnet::appenv({'environment.remotenavmap' => 'on'});
  	my $menu=&Apache::lonmenu::reopenmenu();
 	my $navstatus=&Apache::lonmenu::get_nav_status();
 	if ($menu) {
@@ -255,7 +255,9 @@ MENU
 
         # If we found no problems, print a note to that effect.
         if (!$foundDoableProblem) {
-            $r->print("<font size='+2'>All homework assignments have been completed.</font><br /><br />");
+            $r->print("<font size='+2'>"
+                     .&mt("All homework assignments have been completed.")
+                     ."</font><br /><br />");
         }
     } else {
 	&Apache::lonnavmaps::add_linkitem(\%toplinkitems,'firsthomework',
@@ -321,9 +323,9 @@ MENU
     # user knows there was no error.
     if ($renderArgs->{'counter'} == 0) {
         if ($showOnlyHomework) {
-            $r->print("<p><font size='+1'>".&mt("All homework is currently completed").".</font></p>");
+            $r->print("<p><font size='+1'>".&mt("All homework is currently completed.")."</font></p>");
         } else { # both jumpToFirstHomework and normal use the same: course must be empty
-            $r->print("<p><font size='+1'>This course is empty.</font></p>");
+            $r->print("<p><font size='+1'>".&mt("This course is empty.")."</font></p>");
         }
     }
     #my $td=&tv_interval($t0);
