@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Preferences
 #
-# $Id: lonpreferences.pm,v 1.115 2008/03/06 22:27:45 raeburn Exp $
+# $Id: lonpreferences.pm,v 1.124 2008/06/10 14:49:07 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -108,9 +108,14 @@ sub wysiwygchanger {
     }
     my $switchoff=&mt('Disable WYSIWYG editor');
     my $switchon=&mt('Enable WYSIWYG editor');
+    my $warning='';
+    if ($env{'user.adv'}) {
+       $warning.="<p>".&mt("The WYSIWYG editor only supports simple HTML and is in many cases unsuited for advanced authoring. In a number of cases, it may destroy advanced authoring involving LaTeX and script function calls.")."</p>";
+    }
     $r->print(<<ENDLSCREEN);
 <form name="prefs" action="/adm/preferences" method="post">
 <input type="hidden" name="action" value="set_wysiwyg" />
+$warning
 <br />
 <label><input type="radio" name="wysiwyg" value="off" $onselect /> $switchoff</label><br />
 <label><input type="radio" name="wysiwyg" value="on" $offselect /> $switchon</label>
@@ -123,7 +128,7 @@ sub verify_and_change_wysiwyg {
     my $r = shift;
     my $newsetting=$env{'form.wysiwyg'};
     &Apache::lonnet::put('environment',{'wysiwygeditor' => $newsetting});
-    &Apache::lonnet::appenv('environment.wysiwygeditor' => $newsetting);
+    &Apache::lonnet::appenv({'environment.wysiwygeditor' => $newsetting});
     $r->print('<p>'.&mt('Setting WYSIWYG editor to:').' '.&mt($newsetting).'</p>');
 }
 
@@ -167,7 +172,7 @@ sub verify_and_change_languages {
     my $message='';
     if ($newlanguage) {
         &Apache::lonnet::put('environment',{'languages' => $newlanguage});
-        &Apache::lonnet::appenv('environment.languages' => $newlanguage);
+        &Apache::lonnet::appenv({'environment.languages' => $newlanguage});
         $message=&mt('Set new preferred languages to ').'<tt>"'.$newlanguage.'"</tt>.';
     } else {
         &Apache::lonnet::del('environment',['languages']);
@@ -189,35 +194,45 @@ sub texenginechanger {
     my %userenv = &Apache::lonnet::get('environment',['texengine']);
     my $texengine=$userenv{'texengine'};
 
-    my $pref=&mt('Preferred method to display Math');
     my %mathchoices=('' => 'Default',
-		     'tth' => 'TeX to HTML',
+		     'tth' => 'tth (TeX to HTML)',
 		     #'ttm' => 'TeX to MathML',
 		     'jsMath' => 'jsMath',
-		     'mimetex' => 'Convert to Images'
+		     'mimetex' => 'mimetex (Convert to Images)'
                      );
     my $selectionbox=&Apache::loncommon::select_form($texengine,'texengine',
 						     %mathchoices);
     my $jsMath_start=&Apache::lontexconvert::jsMath_header();
-    my $change=&mt('Change');
-    $r->print(<<ENDLSCREEN);
-<br />
+    my %lt=&Apache::lonlocal::texthash(
+      'headline' => 'Change Math Preferences',
+      'preftxt'  => 'Preferred method to display Math',
+      'change'   => 'Change',
+      'exmpl'    => 'Examples',
+      'jsmath'   => 'jsMath:',
+      'tth'      => 'tth (TeX to HTML):',
+      'mimetex'  => 'mimetex (Convert to Images):',
+    );
 
+    $r->print(<<ENDLSCREEN);
+<h2>$lt{'headline'}</h2>
 <form name="prefs" action="/adm/preferences" method="post">
 <input type="hidden" name="action" value="verify_and_change_texengine" />
-<p>$pref: $selectionbox</p>
-<p><input type="submit" value="$change" /></p>
-</form>
-Examples:
-<p> TeX to HTML <br /> 
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=tth" width="400" hieght="200"></iframe>
+<p>
+$lt{'preftxt'}:<br />
+$selectionbox <input type="submit" value="$lt{'change'}" />
 </p>
-<p>jsMath <br /> 
+</form>
+<br />
+<hr />
+$lt{'exmpl'}
+
+<h3>$lt{'jsmath'}</h3> 
+<p>
 $jsMath_start
 <script type="text/javascript">
 if (jsMath.nofonts == 1) {
     document.writeln
-        ('<center><div style="padding: 10; border-style: solid; border-width:3;'
+        ('<div style="padding: 10; border-style: solid; border-width:3;'
 	 +' border-color: #DD0000; background-color: #FFF8F8; width: 75%; text-align: left">'
 	 +'<small><font color="#AA0000"><b>Warning:</b> '
 	 +'It looks like you don\\\'t have the TeX math fonts installed. '
@@ -227,15 +242,20 @@ if (jsMath.nofonts == 1) {
 	 +'needed fonts.  In the meantime, jsMath will do the best it can '
 	 +'with the fonts you have, but it may not be pretty and some equations '
 	 +'may not be rendered correctly. '
-	 +'</font></small></div></center>');
+	 +'</font></small></div>');
 }
 </script>
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=jsMath" width="400" hieght="200"></iframe>
-
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=jsMath" width="400" height="120"></iframe>
 </p>
-<p> Convert to Images <br />
-<br />
-<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=mimetex" width="400" hieght="200"></iframe>
+
+<h3>$lt{'mimetex'}</h3>
+<p>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=mimetex" width="400" height="100"></iframe>
+</p>
+
+<h3>$lt{'tth'}</h3>
+<p>
+<iframe src="/res/adm/pages/math_example.tex?inhibitmenu=yes&texengine=tth" width="400" height="200"></iframe>
 </p>
 ENDLSCREEN
     if ($env{'environment.texengine'} ne 'jsMath') {
@@ -252,16 +272,16 @@ sub verify_and_change_texengine {
     my $newtexengine  = $env{'form.texengine'};
     $newtexengine=~s/[^\-\w]//g;
     if ($newtexengine eq 'ttm') {
-	&Apache::lonnet::appenv('browser.mathml' => 1);
+	&Apache::lonnet::appenv({'browser.mathml' => 1});
     } else {
 	if ($env{'environment.texengine'} eq 'ttm') {
-	    &Apache::lonnet::appenv('browser.mathml' => 0);
+	    &Apache::lonnet::appenv({'browser.mathml' => 0});
 	}
     }
     my $message='';
     if ($newtexengine) {
         &Apache::lonnet::put('environment',{'texengine' => $newtexengine});
-        &Apache::lonnet::appenv('environment.texengine' => $newtexengine);
+        &Apache::lonnet::appenv({'environment.texengine' => $newtexengine});
         $message=&mt('Set new preferred math display to ').'<tt>"'.$newtexengine.'"</tt>.';
     } else {
         &Apache::lonnet::del('environment',['texengine']);
@@ -400,7 +420,7 @@ sub verify_and_change_rolespref {
     my $message='<hr />';
     if ($hotlist_flag) {
         &Apache::lonnet::put('environment',{'recentroles' => $hotlist_flag});
-        &Apache::lonnet::appenv('environment.recentroles' => $hotlist_flag);
+        &Apache::lonnet::appenv({'environment.recentroles' => $hotlist_flag});
         $message=&mt('Recent '.$role.'s Hotlist is Enabled');
     } else {
         &Apache::lonnet::del('environment',['recentroles']);
@@ -409,7 +429,7 @@ sub verify_and_change_rolespref {
     }
     if ($hotlist_n) {
         &Apache::lonnet::put('environment',{'recentrolesn' => $hotlist_n});
-        &Apache::lonnet::appenv('environment.recentrolesn' => $hotlist_n);
+        &Apache::lonnet::appenv({'environment.recentrolesn' => $hotlist_n});
         if ($hotlist_flag) {
             $message.="<br />".
 		&mt('Display [_1] Most Recent '.$role.'s',$hotlist_n)."\n";
@@ -495,7 +515,7 @@ sub verify_and_change_screenname {
     my $message='';
     if ($newscreen) {
         &Apache::lonnet::put('environment',{'screenname' => $newscreen});
-        &Apache::lonnet::appenv('environment.screenname' => $newscreen);
+        &Apache::lonnet::appenv({'environment.screenname' => $newscreen});
         $message=&mt('Set new screenname to ').'<tt>"'.$newscreen.'."</tt>.';
     } else {
         &Apache::lonnet::del('environment',['screenname']);
@@ -508,7 +528,7 @@ sub verify_and_change_screenname {
     $newscreen=~s/[^ \w]//g;
     if ($newscreen) {
         &Apache::lonnet::put('environment',{'nickname' => $newscreen});
-        &Apache::lonnet::appenv('environment.nickname' => $newscreen);
+        &Apache::lonnet::appenv({'environment.nickname' => $newscreen});
         $message.=&mt('Set new nickname to ').'<tt>"'.$newscreen.'"</tt>.';
     } else {
         &Apache::lonnet::del('environment',['nickname']);
@@ -563,7 +583,7 @@ sub verify_and_change_icons {
     my $newicons  = $env{'form.menumode'};
 
     &Apache::lonnet::put('environment',{'icons' => $newicons});
-    &Apache::lonnet::appenv('environment.icons' => $newicons);
+    &Apache::lonnet::appenv({'environment.icons' => $newicons});
     $r->print(&mt('Set menu mode to [_1].',$newicons));
 }
 
@@ -605,10 +625,84 @@ sub verify_and_change_clicker {
     $newclickers=~s/^\,//;
     $newclickers=~s/\,$//;
     &Apache::lonnet::put('environment',{'clickers' => $newclickers});
-    &Apache::lonnet::appenv('environment.clickers' => $newclickers);
+    &Apache::lonnet::appenv({'environment.clickers' => $newclickers});
     $r->print(&mt('Registering clickers: [_1]',$newclickers));
 }
 
+################################################################
+#               Domcoord Access Subroutines                    #
+################################################################
+
+sub domcoordchanger {
+    my $r = shift;
+    my $user       = $env{'user.name'};
+    my $domain     = $env{'user.domain'};
+    my %userenv = &Apache::lonnet::get
+        ('environment',['domcoord.author']);
+    my $constchecked='';
+    if ($userenv{'domcoord.author'} eq 'blocked') {
+       $constchecked='checked="checked"';
+    }
+    my $text=&mt('By default, the Domain Coordinator can enter your construction space.');
+    my $construction=&mt('Block access to construction space');
+    my $change=&mt('Change');
+    $r->print(<<ENDSCREEN);
+<form name="prefs" action="/adm/preferences" method="post">
+<input type="hidden" name="action" value="verify_and_change_domcoord" />
+$text<br />
+<label><input type="checkbox" name="construction" $constchecked />$construction</label><br />
+<input type="submit" value="$change" />
+</form>
+ENDSCREEN
+}
+
+sub verify_and_change_domcoord {
+    my $r = shift;
+    my $user       = $env{'user.name'};
+    my $domain     = $env{'user.domain'};
+    my %domcoord=('domcoord.author' => '');
+    if ($env{'form.construction'}) { $domcoord{'domcoord.author'}='blocked'; }
+    &Apache::lonnet::put('environment',\%domcoord);
+    &Apache::lonnet::appenv({'environment.domcoord.author' => $domcoord{'domcoord.author'}});
+    $r->print(&mt('Registering Domain Coordinator access restrictions.'));
+}
+
+#################################################################
+##                      Lock Subroutines                        #
+#################################################################
+
+sub lockwarning {
+    my $r = shift;
+    my $title=&mt('Action locked');
+    my $texttop=&mt('LON-CAPA is currently performing the following actions:');
+    my $textbottom=&mt('Changing roles or logging out may result in data corruption.');
+    my ($num,%which)=&Apache::lonnet::get_locks();
+    my $which='';
+    foreach my $id (keys %which) {
+       $which.='<li>'.$which{$id}.'</li>';
+    }
+    my $change=&mt('Override');
+    $r->print(<<ENDSCREEN);
+<form name="prefs" action="/adm/preferences" method="post">
+<input type="hidden" name="action" value="verify_and_change_locks" />
+<h1>$title</h1>
+$texttop
+<ul>
+$which
+</ul>
+$textbottom
+<input type="submit" value="$change" />
+</form>
+ENDSCREEN
+}
+
+sub verify_and_change_lockwarning {
+    my $r = shift;
+    &Apache::lonnet::remove_all_locks();
+    $r->print(&mt('Cleared locks.'));
+}
+
+
 ################################################################
 #         Message Forward                                      #
 ################################################################
@@ -841,7 +935,7 @@ sub verify_and_change_msgforward {
     $newscreen=~s/\,$//;
     if ($newscreen) {
         &Apache::lonnet::put('environment',{'msgforward' => $newscreen});
-        &Apache::lonnet::appenv('environment.msgforward' => $newscreen);
+        &Apache::lonnet::appenv({'environment.msgforward' => $newscreen});
         $message .= &mt('Set message forwarding to ').'<tt>"'.$newscreen.'"</tt>.'
                     .'<br />';
     } else {
@@ -882,7 +976,7 @@ sub verify_and_change_msgforward {
     $notify_with_html =~ s/\s//gs;
     if ($notification) {
         &Apache::lonnet::put('environment',{'notification' => $notification});
-        &Apache::lonnet::appenv('environment.notification' => $notification);
+        &Apache::lonnet::appenv({'environment.notification' => $notification});
         $message.=&mt('Set non-critical message notification address(es) to ').'<tt>"'.$notification.'"</tt>.<br />';
     } else {
         &Apache::lonnet::del('environment',['notification']);
@@ -891,7 +985,7 @@ sub verify_and_change_msgforward {
     }
     if ($critnotification) {
         &Apache::lonnet::put('environment',{'critnotification' => $critnotification});
-        &Apache::lonnet::appenv('environment.critnotification' => $critnotification);
+        &Apache::lonnet::appenv({'environment.critnotification' => $critnotification});
         $message.=&mt('Set critical message notification address(es) to ').'<tt>"'.$critnotification.'"</tt>.<br />';
     } else {
         &Apache::lonnet::del('environment',['critnotification']);
@@ -901,7 +995,7 @@ sub verify_and_change_msgforward {
     if ($critnotification || $notification) {
         if ($notify_with_html) {
             &Apache::lonnet::put('environment',{'notifywithhtml' => $notify_with_html});
-            &Apache::lonnet::appenv('environment.notifywithhtml' => $notify_with_html);
+            &Apache::lonnet::appenv({'environment.notifywithhtml' => $notify_with_html});
             $message.=&mt('Set address(es) to receive excerpts with html retained ').'<tt>"'.$notify_with_html.'"</tt>.';
         } else {
             &Apache::lonnet::del('environment',['notifywithhtml']);
@@ -1017,7 +1111,7 @@ sub verify_and_change_colors {
         my $entry='color.'.$function.'.'.$item;
 	if (($color=~/^\#[0-9A-Fa-f]{6}$/) && (!$env{'form.resetall'})) {
 	    &Apache::lonnet::put('environment',{$entry => $color});
-	    &Apache::lonnet::appenv('environment.'.$entry => $color);
+	    &Apache::lonnet::appenv({'environment.'.$entry => $color});
 	    $message.=&mt('Set '.$colortypes{$item}.' to ').'<tt>"'.$color.'"</tt>.<br />';
 	} else {
 	    &Apache::lonnet::del('environment',[$entry]);
@@ -1027,7 +1121,7 @@ sub verify_and_change_colors {
     }
     my $now = time;
     &Apache::lonnet::put('environment',{'color.timestamp' => $now});
-    &Apache::lonnet::appenv('environment.color.timestamp' => $now);
+    &Apache::lonnet::appenv({'environment.color.timestamp' => $now});
 
     $r->print(<<ENDVCCOL);
 $message
@@ -1325,6 +1419,13 @@ ENDERROR
     #
     if ($caller eq 'reset_by_email') {
         my %data = &Apache::lonnet::tmpget($mailtoken);
+        if (keys(%data) == 0) {
+            &passwordchanger($r,
+                         '<span class="LC_error">'.
+                         &mt('Could not verify current authentication.').'  '.
+                         &mt('Please try again.').'</span>',$caller,$mailtoken);
+            return 1;
+        }
         if ($currentpass ne $data{'temppasswd'}) {
             &passwordchanger($r,
                          '<span class="LC_error">'.
@@ -1499,7 +1600,7 @@ sub verify_and_change_discussion {
         if ($newdisp eq 'unread') {
             $message .=&mt('In discussions: only new posts will be displayed.').'<br />';
             &Apache::lonnet::put('environment',{'discdisplay' => $newdisp});
-            &Apache::lonnet::appenv('environment.discdisplay' => $newdisp);
+            &Apache::lonnet::appenv({'environment.discdisplay' => $newdisp});
         } else {
             $message .= &mt('In discussions: all posts will be displayed.').'<br />';
             &Apache::lonnet::del('environment',['discdisplay']);
@@ -1511,7 +1612,7 @@ sub verify_and_change_discussion {
         if ($newmark eq 'ondisp') {
            $message.=&mt('In discussions: new posts will be cease to be identified as "NEW" after display.').'<br />';
             &Apache::lonnet::put('environment',{'discmarkread' => $newmark});
-            &Apache::lonnet::appenv('environment.discmarkread' => $newmark);
+            &Apache::lonnet::appenv({'environment.discmarkread' => $newmark});
         } else {
             $message.=&mt('In discussions: posts will be identified as "NEW" until marked as read by the reader.').'<br />';
             &Apache::lonnet::del('environment',['discmarkread']);
@@ -1576,7 +1677,7 @@ sub verify_and_change_coursepage {
     if ($newdisp eq 'firstres') {
         $message .= $lt{'ywbt'}.'<br />';
         &Apache::lonnet::put('environment',{'course_init_display' => $newdisp});
-        &Apache::lonnet::appenv('environment.course_init_display' => $newdisp);
+        &Apache::lonnet::appenv({'environment.course_init_display' => $newdisp});
     } else {
         $message .= $lt{'apwb'}.'<br />';
         &Apache::lonnet::del('environment',['course_init_display']);
@@ -1818,7 +1919,7 @@ sub handler {
                       href     => '/adm/preferences',
                       subroutine => \&clickerchanger,
                       breadcrumb =>
-                          { href => '/adm/preferences?action=changeicons',
+                          { href => '/adm/preferences?action=changeclicker',
                             text => 'Register Clicker'},
                       },
                     { action   => 'verify_and_change_clicker',
@@ -1828,6 +1929,37 @@ sub handler {
                             text => 'Register Clicker'},
                       printmenu => 'yes',
                       }));
+   if ($env{'user.adv'}) {
+      push (@Options,({ action   => 'changedomcoord',
+                        linktext => 'Restrict Domain Coordinator Access',
+                        href     => '/adm/preferences',
+                        subroutine => \&domcoordchanger,
+                        breadcrumb =>
+                            { href => '/adm/preferences?action=changedomcoord',
+                              text => 'Restrict Domain Coordinator Access'},
+                      },
+                      { action   => 'verify_and_change_domcoord',
+                        subroutine => \&verify_and_change_domcoord,
+                        breadcrumb =>
+                            { href => '/adm/preferences?action=changedomcoord',
+                              text => 'Restrict Domain Coordinator Access'},
+                        printmenu => 'yes',
+                      }));
+    }
+
+    push (@Options,({ action   => 'lockwarning',
+                      subroutine => \&lockwarning,
+                      breadcrumb =>
+                          { href => '/adm/preferences?action=lockwarning',
+                            text => 'Lock Warnings'},
+                      },
+                    { action   => 'verify_and_change_locks',
+                      subroutine => \&verify_and_change_lockwarning,
+                      breadcrumb =>
+                          { href => '/adm/preferences?action=lockwarning',
+                            text => 'Lockwarnings'},
+                      printmenu => 'yes',
+                      }));
 
 
     if (&Apache::lonnet::allowed('whn',$env{'request.course.id'})
@@ -1849,7 +1981,7 @@ sub handler {
                        }));
     }
 
-    if ($env{'user.name'} =~ /^(albertel|fox|foxr|koretemey|korte|hallmat3|turtle|raeburn)$/) {
+    if ($env{'user.name'} =~ /^(albertel|fox|foxr|kortemey|korte|raeburn)$/) {
         push (@Options,({ action => 'debugtoggle',
                           printmenu => 'yes',
                           subroutine => \&toggle_debug,
@@ -1932,7 +2064,7 @@ sub toggle_debug {
     if ($env{'user.debug'}) {
         &Apache::lonnet::delenv('user\.debug');
     } else {
-        &Apache::lonnet::appenv('user.debug' => 1);
+        &Apache::lonnet::appenv({'user.debug' => 1});
     }
 }
 
