@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Simple Page Editor
 #
-# $Id: lonsimplepage.pm,v 1.55 2008/01/28 10:37:33 foxr Exp $
+# $Id: lonsimplepage.pm,v 1.56 2008/03/20 12:06:51 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -216,7 +216,9 @@ sub handler {
 
     if ($allowed) {
 	$r->print('<p>'.
-		  &Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'<br /><a href="'.$r->uri.'?forcestudent=1'.$refarg.'"><font size="+1">'.&mt('Show Student View').'</font></a>'.
+		  &Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes',&mt('Help with filling in text boxes'))
+                 .'<br /><a href="'.$r->uri.'?forcestudent=1'.$refarg.'">'
+                 .'<font size="+1">'.&mt('Show Student View').'</font></a>'.
 		  &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
     } elsif ($privileged and $target ne 'tex') {
         my $edittext = &mt('Edit');
@@ -283,9 +285,9 @@ sub handler {
 	    $r->print(
 		      '<form method="post" enctype="multipart/form-data">'.
 		      '<input type="hidden" name="forceedit" value="edit" />'.
-		      '<h3>Upload a Photo</h3>'.
+		      '<h3>'.&mt('Upload a Photo').'</h3>'.
 		      '<input type="file" name="uploaddoc" size="50" />'.
-		      '<input type="submit" name="storeupl" value="Upload" />'.
+		      '<input type="submit" name="storeupl" value="'.&mt('Upload').'" />'.
 		      '</form><form method="post">'.
                       '<input type="hidden" name="forceedit" value="edit" />');
 	}
@@ -318,9 +320,9 @@ sub handler {
                     if ($allowed) {
                         if ($env{'form.grade_target'} ne 'tex') {
                             $r->print(
-                                      '<br />Title<br /><textarea cols="80" rows="2" name="'.$field.'">'.
+                                      '<br /><h3>'.&mt('Title').'</h3><textarea cols="80" rows="2" name="'.$field.'">'.
                                       &HTML::Entities::encode($syllabus{$field},'"&<>').
-                                      '</textarea><input type="submit" name="storesyl" value="Save" />');
+                                      '</textarea><input type="submit" name="storesyl" value="'.&mt('Save').'" />');
                         } else {
                             my $safeinit;
                             $r->print(&Apache::lonxml::xmlparse($r,'tex',$syllabus{$field},$safeinit));
@@ -346,7 +348,7 @@ sub handler {
 			if ($target ne 'tex') {
 			    $r->print('<br /><textarea cols="80" rows="24" name="'.$field.'" id="'.$field.'">'.
                                       &HTML::Entities::encode($syllabus{$field},'"&<>').
-				      '</textarea><input type="submit" name="storesyl" value="Save" />');
+				      '</textarea><input type="submit" name="storesyl" value="'.&mt('Save').'" />');
 			} else {
 			    my $safeinit;
 			    $r->print(&Apache::lonxml::xmlparse($r,'tex',$syllabus{$field},$safeinit));
