@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # portfolio browser
 #
-# $Id: portfolio.pm,v 1.190 2008/02/05 05:54:59 raeburn Exp $
+# $Id: portfolio.pm,v 1.195 2008/06/16 23:34:12 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -93,7 +93,9 @@ sub display_common {
 					 'Upload file to current directory:',
 					 'createdir' => 'Create Subdirectory',
 					 'createdir_label' => 
-					 'Create subdirectory in current directory:');
+					 'Create subdirectory in current directory:',
+                                         'parse' => 'If HTML file, upload embedded images/multimedia/css/linked files'
+                                            );
         my $escuri = &HTML::Entities::encode($r->uri,'&<>"');
 	my $help_fileupload = &Apache::loncommon::help_open_topic('Portfolio AddFiles');
 	my $help_createdir = &Apache::loncommon::help_open_topic('Portfolio CreateDirectory');
@@ -101,6 +103,17 @@ sub display_common {
 	# FIXME: This line should be deleted once Portfolio uses breadcrumbs
 	$r->print(&Apache::loncommon::help_open_topic('Portfolio About', &mt('Help on the portfolio')));
 
+        my $parse_check;
+        if (!&suppress_embed_prompt()) {
+            $parse_check = <<"END";
+        <br />
+        <span class="LC_nobreak">
+         <label>$text{'parse'}
+         <input type="checkbox" name="parserflag" checked="checked" />
+         </label>
+        </span>
+END
+        }
         $r->print(<<"TABLE"); 
 <table id="LC_portfolio_actions">
   <tr id="LC_portfolio_upload">
@@ -115,7 +128,9 @@ sub display_common {
 	<input type="hidden" name="action" value="$env{"form.action"}" />
 	<input type="hidden" name="fieldname" value="$env{"form.fieldname"}" />
 	<input type="hidden" name="mode" value="$env{"form.mode"}" />
-	<input type="submit" name="storeupl" value="$text{'upload'}" />$help_fileupload
+	<input type="submit" name="storeupl" value="$text{'upload'}" />
+$help_fileupload
+$parse_check
       </form>
     </td>
   </tr>
@@ -183,7 +198,7 @@ sub display_directory_line {
         $r->print('<td><img alt="" src="'.&Apache::loncommon::icon($filename).'" /></td>');
         $r->print('<td>'.$version_flag.'</td>');
     } else { # this is a graded or handed back file
-        my ($user,$domain) = &get_name_dom();
+        my ($user,$domain) = &get_name_dom($env{'form.group'});
         my $permissions_hash = &Apache::lonnet::get_portfile_permissions($domain,$user);
         if (defined($$permissions_hash{$fullpath})) {
             foreach my $array_item (@{$$permissions_hash{$fullpath}}) {
@@ -231,7 +246,7 @@ sub display_directory {
     my $select_mode;
     my $checked_files;
     my $port_path = &get_port_path();
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $access_admin_text = &mt('View Status');
     if ($can_setacl) {
         $access_admin_text = &mt('View/Change Status');
@@ -531,12 +546,12 @@ sub done {
 }
 
 sub delete {
-    my ($r,$url)=@_;
+    my ($r,$url,$group)=@_;
     my @check;
     my $file_name = $env{'form.currentpath'}.$env{'form.selectfile'};
     $file_name = &prepend_group($file_name);
     my @files=&Apache::loncommon::get_env_multiple('form.selectfile');
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     if (&Apache::lonnet::is_locked($file_name,$udom,$uname) eq 'true') {
         $r->print(&mt('The file is locked and cannot be deleted.').'<br />');
         $r->print(&done('Back',$url));
@@ -556,7 +571,7 @@ sub delete_confirmed {
     my ($r,$url,$group)=@_;
     my @files=&Apache::loncommon::get_env_multiple('form.selectfile');
     my $result;
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $port_path = &get_port_path();
     my $current_permissions = &Apache::lonnet::get_portfile_permissions($udom,
                                                                         $uname);
@@ -615,10 +630,10 @@ sub delete_dir {
 } 
 
 sub delete_dir_confirmed {
-    my ($r,$url)=@_;
+    my ($r,$url,$group)=@_;
     my $directory_name = $env{'form.currentpath'};
     $directory_name =~ s|/$||; # remove any trailing slash
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $namespace = &get_namespace();
     my $port_path = &get_port_path();
     my $result=&Apache::lonnet::removeuserfile($uname,$udom,$port_path.
@@ -643,9 +658,9 @@ sub delete_dir_confirmed {
 }
 
 sub rename {
-    my ($r,$url)=@_;
+    my ($r,$url,$group)=@_;
     my $file_name = $env{'form.currentpath'}.$env{'form.rename'};
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     $file_name = &prepend_group($file_name);
     if (&Apache::lonnet::is_locked($file_name,$udom,$uname) eq 'true') {
         $r->print("The file is locked and cannot be renamed.<br />");
@@ -661,7 +676,7 @@ sub rename {
 sub rename_confirmed {
     my ($r,$url,$group)=@_;
     my $filenewname=&Apache::lonnet::clean_filename($env{'form.filenewname'});
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $port_path = &get_port_path();
     if ($filenewname eq '') {
 	$r->print('<span class="LC_error">'.
@@ -745,7 +760,7 @@ sub access_for_renamed {
 
 sub display_access {
     my ($r,$url,$group,$can_setacl,$port_path,$action) = @_;
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $file_name = $env{'form.currentpath'}.$env{'form.access'};
     $file_name = &prepend_group($file_name);
     my $current_permissions = &Apache::lonnet::get_portfile_permissions($udom,
@@ -959,7 +974,7 @@ sub update_access {
     $r->print('<h3>'.&mt('Allowing others to retrieve file: [_1]',
               $port_path.$file_name).'</h3>'."\n");
     $file_name = &prepend_group($file_name);
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my ($errors,$outcome,$deloutcome,$new_values,$translation);
     if ($totalprocessed) {
         ($outcome,$deloutcome,$new_values,$translation) =
@@ -1754,12 +1769,11 @@ sub check_for_upload {
     my ($path,$fname,$group,$element) = @_;
     my $disk_quota = &get_quota($group);
     my $filesize = (length($env{'form.'.$element})) / 1000; #express in k (1024?)
-
     my $portfolio_root = &get_portfolio_root();
     my $port_path = &get_port_path();
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     # Fixme --- Move the checking for existing file to LOND error return
-    my @dir_list=&get_dir_list($portfolio_root,$path);
+    my @dir_list=&get_dir_list($portfolio_root,$path,$group);
     my $found_file = 0;
     my $locked_file = 0;
     foreach my $line (@dir_list) {
@@ -1773,7 +1787,8 @@ sub check_for_upload {
             } 
         }
     }
-    my $current_disk_usage = &Apache::lonnet::diskusage($udom,$uname,$portfolio_root);
+    my $getpropath = 1;
+    my $current_disk_usage = &Apache::lonnet::diskusage($udom,$uname,$portfolio_root,$getpropath);
 
     if (($current_disk_usage + $filesize) > $disk_quota){
         my $msg = '<span class="LC_error">'.
@@ -1801,10 +1816,17 @@ sub check_for_upload {
 sub upload {
     my ($r,$url,$group)=@_;
     my $fname=&Apache::lonnet::clean_filename($env{'form.uploaddoc.filename'});
-
-    my ($state,$msg) = &check_for_upload($env{'form.currentpath'},
-					 $fname,$group,'uploaddoc');
-
+    my $disk_quota = &get_quota($group);
+    my $portfolio_root = &get_portfolio_root();
+    my $port_path = &get_port_path();
+    my ($uname,$udom) = &get_name_dom($group);
+    my $getpropath = 1;
+    my $current_disk_usage = &Apache::lonnet::diskusage($udom,$uname,$portfolio_root,$getpropath);
+    my ($state,$msg) = 
+        &Apache::loncommon::check_for_upload($env{'form.currentpath'},$fname,
+		                             $group,'uploaddoc',$portfolio_root,
+                                             $port_path,$disk_quota,
+                                             $current_disk_usage,$uname,$udom);
     if ($state eq 'will_exceed_quota'
 	|| $state eq 'file_locked'
 	|| $state eq 'file_exists' ) {
@@ -1812,11 +1834,11 @@ sub upload {
 	return;
     }
 
-    my $port_path = &get_port_path();
     my (%allfiles,%codebase,$mode);
     if ($env{'form.uploaddoc.filename'} =~ m/(\.htm|\.html|\.shtml)$/i) {
-	$mode = 'parse';
-	
+        if ($env{'form.parserflag'}) {
+	    $mode = 'parse';
+        }
     }
     my $result=
 	&Apache::lonnet::userfileupload('uploaddoc','',
@@ -1828,76 +1850,30 @@ sub upload {
 	$r->print(&done('Back',$url));
     } else {
 	if (%allfiles) {
-	    my $state = <<STATE;
+            if (!&suppress_embed_prompt()) {
+	        my $state = <<STATE;
     <input type="hidden" name="action"      value="upload_embedded" />
     <input type="hidden" name="currentpath" value="$env{'form.currentpath'}" />
     <input type="hidden" name="fieldname"   value="$env{'form.fieldname'}" />
     <input type="hidden" name="mode"        value="$env{'form.mode'}" />
 STATE
-            $r->print("<h2>".&mt("Reference Warning")."</h2>");
-            $r->print("<p>".&mt("Completed upload of the file. This file contained references to other files. You must upload the referenced files or else the uploaded file may not work properly.")."</p>");
-            $r->print("<p>".&mt("Please select the locations from which the referenced files are to be uploaded.")."</p>");
-	    $r->print(&Apache::londocs::ask_for_embedded_content('/adm/portfolio',$state,\%allfiles,\%codebase,
+                $r->print("<h2>".&mt("Reference Warning")."</h2>");
+                $r->print("<p>".&mt("Completed upload of the file. This file contained references to other files. You must upload the referenced files or else the uploaded file may not work properly.")."</p>");
+                $r->print("<p>".&mt("Please select the locations from which the referenced files are to be uploaded.")."</p>");
+	        $r->print(&Apache::loncommon::ask_for_embedded_content('/adm/portfolio',$state,\%allfiles,\%codebase,
 				      {'error_on_invalid_names'   => 1,
 				       'ignore_remote_references' => 1,}));
-	    $r->print('<p>Or '.&done('Return to directory',$url).'</p>');
+	        $r->print('<p>Or '.&done('Return to directory',$url).'</p>');
+            }
 	} else {
 	    $r->print(&done(undef,$url));
 	}
     }
 }
 
-sub upload_embedded {
-    my ($r,$url,$group)=@_;
-    for (my $i=0; $i<$env{'form.number_embedded_items'}; $i++) {
-	next if (!exists($env{'form.embedded_item_'.$i.'.filename'}));
-	my $orig_uploaded_filename = 
-	    $env{'form.embedded_item_'.$i.'.filename'};
-
-	$env{'form.embedded_orig_'.$i} = 
-	    &unescape($env{'form.embedded_orig_'.$i});
-	my ($path,$fname) = 
-	    ($env{'form.embedded_orig_'.$i} =~ m{(.*/)([^/]*)});
-	# no path, whole string is fname
-	if (!$fname) { $fname = $env{'form.embedded_orig_'.$i} };
-
-	$path = $env{'form.currentpath'}.$path;
-	$fname = &Apache::lonnet::clean_filename($fname);
-
-	my ($state,$msg) = &check_for_upload($path,$fname,$group,
-					     'embedded_item_'.$i);
-
-	if ($state eq 'will_exceed_quota'
-	    || $state eq 'file_locked'
-	    || $state eq 'file_exists' ) {
-	    $r->print($msg);
-	    next;
-	}
-
-	my $port_path = &get_port_path();
-	my $src_path = $env{'form.embedded_orig_'.$i};
-	$env{'form.embedded_item_'.$i.'.filename'}=$fname;
-	
-	my $result=
-	    &Apache::lonnet::userfileupload('embedded_item_'.$i,'',
-					    $port_path.$path);
-	if ($result !~ m|^/uploaded/|) {
-	    $r->print('<span class="LC_error">'
-                      .&mt('An error occurred ([_1]) while trying to upload [_2] for embedded element [_3].'
-                           ,$result,$orig_uploaded_filename,$env{'form.embedded_orig_'.$i})
-                      .'</span><br />');
-	    next;
-	} else {
-	    $r->print("<p> Uploaded ".
-		      &display_file($port_path.$path,$fname).'</p>');
-	}
-    }
-    $r->print(&done(undef,$url));
-}
-
 sub lock_info {
     my ($r,$url,$group) = @_;
-    my ($uname,$udom) = &get_name_dom();
+    my ($uname,$udom) = &get_name_dom($group);
     my $current_permissions = &Apache::lonnet::get_portfile_permissions($udom,
                                                                        $uname);
     my $file_name = $env{'form.lockinfo'};
@@ -1939,7 +1915,7 @@ sub lock_info {
 }
 
 sub createdir {
-    my ($r,$url)=@_;
+    my ($r,$url,$group)=@_;
     my $newdir=&Apache::lonnet::clean_filename($env{'form.newdir'});
     if ($newdir eq '') {
     	$r->print('<span class="LC_error">'.
@@ -1949,7 +1925,7 @@ sub createdir {
 	    return;
     }
     my $portfolio_root = &get_portfolio_root(); 
-    my @dir_list=&get_dir_list($portfolio_root);
+    my @dir_list=&get_dir_list($portfolio_root,undef,$group);
     my $found_file = 0;
     foreach my $line (@dir_list) {
         my ($filename)=split(/\&/,$line,2);
@@ -1962,7 +1938,7 @@ sub createdir {
                       .&mt('Unable to create a directory named [_1].','<strong>'.$newdir.'</strong>')
                       .' '.&mt('A file or directory by that name already exists.').'</span><br />');
     } else {
-        my ($uname,$udom) = &get_name_dom();
+        my ($uname,$udom) = &get_name_dom($group);
         my $port_path = &get_port_path();
         my $result=&Apache::lonnet::mkdiruserfile($uname,$udom,
 	         $port_path.$env{'form.currentpath'}.$newdir);
@@ -1983,7 +1959,7 @@ sub createdir {
 sub get_portfolio_root {
     my ($udom,$uname,$group) = @_;
     if (!(defined($udom)) || !(defined($uname))) {
-        ($uname,$udom) = &get_name_dom();
+        ($uname,$udom) = &get_name_dom($group);
     }
     my $path = '/userfiles/portfolio';
     if (!defined($group)) { 
@@ -1994,7 +1970,7 @@ sub get_portfolio_root {
     if (defined($group)) {
         $path = '/userfiles/groups/'.$group.'/portfolio';
     } 
-    return (&Apache::loncommon::propath($udom,$uname).$path);
+    return $path;
 }
 
 sub get_group_quota {
@@ -2009,21 +1985,23 @@ sub get_group_quota {
         $group_quota = $group_info{'quota'}; #expressed in Mb
         if ($group_quota) {
             $group_quota = 1000 * $group_quota; #expressed in k
-        } 
+        }
     }
     return $group_quota;
-} 
+}
 
 sub get_dir_list {
-    my ($portfolio_root,$path) = @_;
+    my ($portfolio_root,$path,$group) = @_;
     $path ||= $env{'form.currentpath'};
-    my ($uname,$udom) = &get_name_dom();
-    return &Apache::lonnet::dirlist($path,$udom,$uname,$portfolio_root);
+    my ($uname,$udom) = &get_name_dom($group);
+    my $getpropath = 1;
+    return &Apache::lonnet::dirlist($portfolio_root.$path,$udom,$uname,$getpropath);
 }
 
 sub get_name_dom {
+    my ($group) = @_;
     my ($uname,$udom);
-    if (defined($env{'form.group'})) {
+    if (defined($group)) {
         $udom = $env{'course.'.$env{'request.course.id'}.'.domain'};
         $uname = $env{'course.'.$env{'request.course.id'}.'.num'};
     } else {
@@ -2044,7 +2022,7 @@ sub prepend_group {
 sub get_namespace {
     my $namespace = 'portfolio';
     if (defined($env{'form.group'})) {
-        my ($uname,$udom) = &get_name_dom();
+        my ($uname,$udom) = &get_name_dom($env{'form.group'});
         $namespace .= '_'.$udom.'_'.$uname.'_'.$env{'form.group'};
     }
     return $namespace;
@@ -2132,6 +2110,17 @@ sub get_quota {
     return $disk_quota;
 }
 
+sub suppress_embed_prompt {
+    my $suppress_prompt = 0;
+    if (($env{'request.role'} =~ /^st/) && ($env{'request.course.id'} ne '')) {
+        if ($env{'course.'.$env{'request.course.id'}.'.suppress_embed_prompt'} eq 'yes') {
+            $suppress_prompt = 1;
+        }
+    }
+    return $suppress_prompt;
+}
+
+
 sub handler {
     # this handles file management
     my $r = shift;
@@ -2158,8 +2147,8 @@ sub handler {
            &Apache::lonnet::allowed('vcg',$env{'request.course.id'}.($env{'request.course.sec'}?'/'.$env{'request.course.sec'}:''));
         $env{'form.group'} =~ s/\W//g;
 	$group = $env{'form.group'};
-        if ($group) {
-            ($uname,$udom) = &get_name_dom();
+        if ($group ne '') {
+            ($uname,$udom) = &get_name_dom($group);
             my %curr_groups = &Apache::longroup::coursegroups($udom,$uname,
 							       $group); 
             if (%curr_groups) {
@@ -2256,7 +2245,14 @@ sub handler {
         }
     } elsif ($env{'form.action'} eq 'upload_embedded') {
 	if ($can_upload) {
-	    &upload_embedded($r,$url,$group);
+            my $disk_quota = &get_quota($group);
+            my $getpropath = 1;
+            my $current_disk_usage = 
+                &Apache::lonnet::diskusage($udom,$uname,$portfolio_root,$getpropath);
+	    $r->print(
+                &Apache::loncommon::upload_embedded('portfolio',$port_path,$uname,$udom,
+                    $group,$portfolio_root,$group,$disk_quota,$current_disk_usage));
+            $r->print(&done(undef,$url));
         } else {
             &missing_priv($r,$url,'upload');
         }
@@ -2268,13 +2264,13 @@ sub handler {
         }
     } elsif ($env{'form.action'} eq 'delete') {
         if ($can_delete) {
-	    &delete($r,$url);
+	    &delete($r,$url,$group);
         } else {
             &missing_priv($r,$url,'delete');
         }
     } elsif ($env{'form.action'} eq 'deletedir' && $env{'form.confirmed'}) {
         if ($can_delete) {
-	    &delete_dir_confirmed($r,$url);
+	    &delete_dir_confirmed($r,$url,$group);
         } else {
             &missing_priv($r,$url,'delete');
         }
@@ -2294,7 +2290,7 @@ sub handler {
         $env{'form.selectfile'} = $env{'form.rename'};
         $env{'form.action'} = 'rename';
         if ($can_modify) {
-	    &rename($r,$url);
+	    &rename($r,$url,$group);
         } else {
             &missing_priv($r,$url,'rename');
         }
@@ -2319,7 +2315,7 @@ sub handler {
         }
     } elsif ($env{'form.createdir'}) {
         if ($can_upload) {
-	    &createdir($r,$url);
+	    &createdir($r,$url,$group);
         } else {
             &missing_priv($r,$url,'upload');
         }
@@ -2334,7 +2330,7 @@ sub handler {
             &Apache::lonhtmlcommon::clear_breadcrumbs();
             $r->print(&coursegrp_portfolio_header($udom,$uname,$grp_desc));
         }
-        my @dir_list=&get_dir_list($portfolio_root);
+        my @dir_list=&get_dir_list($portfolio_root,$current_path,$group);
 	if ($dir_list[0] eq 'no_such_dir'){
 	    # two main reasons for this:
             #    1) never been here, so directory structure not created
@@ -2350,8 +2346,7 @@ sub handler {
 		$current_path = '/'; # force it back to the root        
 	    }
 	    # now grab the directory list again, for the first time
-	    @dir_list=&Apache::lonnet::dirlist($current_path,
-					    $udom,$uname,$portfolio_root);
+            @dir_list=&get_dir_list($portfolio_root,$current_path,$group);
         }
 	# need to know if directory is empty so it can be removed if desired
 	my $is_empty=(@dir_list == 2);
