@@ -30,7 +30,6 @@ use strict;
 use Apache::Constants qw(:common :http);
 use Apache::lonrep();
 use Apache::lonnet;
-use CGI::Cookie();
 use LONCAPA();
 
 sub handler {
@@ -59,10 +58,9 @@ sub handler {
             return OK;
         }
     }
-    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
-    my $lonid=$cookies{'lonID'};
-    if ($lonid) {
-        $r->log_reason('Copying failed in lonuploadrep for '.$lonid->value);
+    my $handle = &Apache::lonnet::check_for_valid_session($r);
+    if ($handle) {
+        $r->log_reason('Copying failed in lonuploadrep for '.$handle);
         return HTTP_NOT_FOUND;
     } else {
         $r->log_reason('Copying failed in lonuploadrep for unknown user'); 
