@@ -9,7 +9,7 @@
 #  and displays a page showing the results of the action.
 #
 #
-# $Id: loncfile.pm,v 1.87 2007/10/18 21:58:21 albertel Exp $
+# $Id: loncfile.pm,v 1.89 2008/05/27 17:33:28 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -109,6 +109,17 @@ sub Debug {
     }
 }
 
+sub done {
+   my ($url)=@_;
+   my $done=&mt("Done");
+   return(<<ENDDONE);
+<a href="$url">$done</a>
+<script type="text/javascript">
+location.href="$url";
+</script>
+ENDDONE
+}
+
 =pod
 
 =item URLToPath($url)
@@ -435,7 +446,7 @@ Parameters:
 
 sub CloseForm2 {
     my ($request, $user, $fn) = @_;
-    $request->print('<h3><a href="'.&url($fn).'/">'.&mt('Done').'</a></h3>');
+    $request->print(&done(&url($fn)));
 }
 
 =pod
@@ -1128,8 +1139,8 @@ sub NewDir2 {
 
 sub decompress2 {
     my ($r, $user, $dir, $file) = @_;
-    &Apache::lonnet::appenv('cgi.file' => $file);
-    &Apache::lonnet::appenv('cgi.dir' => $dir);
+    &Apache::lonnet::appenv({'cgi.file' => $file});
+    &Apache::lonnet::appenv({'cgi.dir' => $dir});
     my $result=&Apache::lonnet::ssi_body('/cgi-bin/decompress.pl');
     $r->print($result);
     &Apache::lonnet::delenv('cgi.file');
@@ -1256,7 +1267,7 @@ sub phasetwo {
             $r->print('<h3><a href="'.&url($dest).'">'.&mt('Return to Directory').'</a></h3>');
             $r->print('<h3><a href="'.&url($dest_newname).'">'.$disp_newname.'</a></h3>');
         } else {
-	    $r->print('<h3><a href="'.&url($dest).'">'.&mt('Done').'</a></h3>');
+	    $r->print(&done(&url($dest)));
 	}
     }
 }
