@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Publication Handler
 #
-# $Id: lonpublisher.pm,v 1.232 2008/03/18 23:31:31 raeburn Exp $
+# $Id: lonpublisher.pm,v 1.239 2008/06/30 18:10:24 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -143,6 +143,8 @@ my $cudom;
 my $registered_cleanup;
 my $modified_urls;
 
+my $lock;
+
 =pod
 
 =item B<metaeval>
@@ -272,7 +274,7 @@ sub metaread {
     }
     &metaeval($metastring,$prefix);
     return '<br /><b>'.&mt('Processed file').':</b> <tt>'.
-	&Apache::loncfile::display($fn).'</tt>';
+	&Apache::loncfile::display($fn).'</tt><br />';
 }
 
 #########################################
@@ -329,9 +331,9 @@ sub textfield {
     $value=~s/\s+/ /gs;
     $title=&mt($title);
     $env{'form.'.$name}=$value;
-    return "\n<p><font color=\"#800000\" face=\"helvetica\"><b>$title:".
-           "</b></font></p><br />".
-           '<input type="text" name="'.$name.'" size=80 value="'.$value.'" />';
+    return "\n".&Apache::lonhtmlcommon::row_title($title)
+           .'<input type="text" name="'.$name.'" size="80" value="'.$value.'" />'
+           .&Apache::lonhtmlcommon::row_closure();
 }
 
 sub text_with_browse_field {
@@ -341,12 +343,16 @@ sub text_with_browse_field {
     $value=~s/\s+/ /gs;
     $title=&mt($title);
     $env{'form.'.$name}=$value;
-    return "\n<p><font color=\"#800000\" face=\"helvetica\"><b>$title:".
-           "</b></font></p><br />".
-           '<input type="text" name="'.$name.'" size=80 value="'.$value.'" />'.
-	   '<a href="javascript:openbrowser(\'pubform\',\''.$name.'\',\''.$restriction.'\');">'.&mt('Select').'</a>&nbsp;'.
-	   '<a href="javascript:opensearcher(\'pubform\',\''.$name.'\');">'.&mt('Search').'</a>';
-	   
+    return "\n".&Apache::lonhtmlcommon::row_title($title)
+          .'<input type="text" name="'.$name.'" size="80" value="'.$value.'" />'
+          .'<br />'
+	  .'<a href="javascript:openbrowser(\'pubform\',\''.$name.'\',\''.$restriction.'\');">'
+          .&mt('Select')
+          .'</a>&nbsp;'
+	  .'<a href="javascript:opensearcher(\'pubform\',\''.$name.'\');">'
+          .&mt('Search')
+          .'</a>'
+          .&Apache::lonhtmlcommon::row_closure();
 }
 
 sub hiddenfield {
@@ -370,8 +376,8 @@ sub selectbox {
     } else {
 	$env{'form.'.$name}=$idlist[0];
     }
-    my $selout="\n<p><font color=\"#800000\" face=\"helvetica\"><b>$title:".
-	'</b></font></p><br /><select name="'.$name.'">';
+    my $selout="\n".&Apache::lonhtmlcommon::row_title($title)
+              .'<select name="'.$name.'">';
     foreach (@idlist) {
         $selout.='<option value=\''.$_.'\'';
         if ($_ eq $value) {
@@ -379,7 +385,8 @@ sub selectbox {
 	}
         else {$selout.='>'.&{$functionref}($_).'</option>';}
     }
-    return $selout.'</select>';
+    $selout.='</select>'.&Apache::lonhtmlcommon::row_closure();
+    return $selout;
 }
 
 sub select_level_form {
@@ -663,6 +670,7 @@ sub fix_ids_and_indices {
                 "Max Index: $maxindex (min 10)\n";
     }
     my $outstring='';
+    my $responsecounter=1;
     my @parser;
     $parser[0]=HTML::LCParser->new(\$content);
     $parser[-1]->xml_mode(1);
@@ -678,6 +686,10 @@ sub fix_ids_and_indices {
 		    next;
 		}
 		if ($lctag eq 'base') { next; }
+                if (($lctag eq 'part') || ($lctag eq 'problem')) {
+                    $responsecounter=0;
+                }
+                if ($lctag=~/response$/) { $responsecounter++; }
 		my %parms=%{$token->[2]};
 		$counter=$addid{$tag};
 		if (!$counter) { $counter=$addid{$lctag}; }
@@ -773,7 +785,7 @@ sub fix_ids_and_indices {
 		}
 		if (!$endtag) { if ($token->[4]=~m:/>$:) { $endtag=' /'; }; }
 		$outstring.='<'.$tag.$newparmstring.$endtag.'>';
-		if ($lctag eq 'm' || $lctag eq 'script' 
+		if ($lctag eq 'm' || $lctag eq 'script' || $lctag eq 'answer' 
                     || $lctag eq 'display' || $lctag eq 'tex') {
 		    $outstring.=&get_all_text_unbalanced('/'.$lctag,\@parser);
 		}
@@ -782,7 +794,12 @@ sub fix_ids_and_indices {
 		    unless ($token->[1] eq 'allow') {
 			$outstring.='</'.$token->[1].'>';
 		    }
-		}
+                }
+                if ((($token->[1] eq 'part') || ($token->[1] eq 'problem'))
+                    && (!$responsecounter)) {
+                    my $outstring='<span class="LC_error">'.&mt('Found [_1] without responses. This resource cannot be published.',$token->[1]).'</span>';
+                    return ($outstring,1);
+                }
 	    } else {
 		$outstring.=$token->[1];
 	    }
@@ -838,8 +855,7 @@ sub store_metadata {
         return ($error,undef);
     }
     my $dbh = &Apache::lonmysql::get_dbh();
-    if (($metadata{'obsolete'}) || ($metadata{'copyright'} eq 'priv') ||
-	($metadata{'copyright'} eq 'custom')) {
+    if (($metadata{'obsolete'}) || ($metadata{'copyright'} eq 'priv')) {
         # remove this entry
 	my $delitem = 'url = '.$dbh->quote($metadata{'url'});
 	$status = &LONCAPA::lonmetadata::delete_metadata($dbh,undef,$delitem);
@@ -952,7 +968,7 @@ sub publish {
 	    print $logfile "Copied original file to ".$copyfile."\n";
         } else {
 	    print $logfile "Unable to write backup ".$copyfile.':'.$!."\n";
-	    return ("<span class=\"LC_error\">Failed to write backup copy, $!,FAIL</span>",1);
+	    return ("<span class=\"LC_error\">".&mt("Failed to write backup copy, [_1], FAIL",$1)."</span>",1);
         }
 # ------------------------------------------------------------- IDs and indices
 	
@@ -1028,7 +1044,7 @@ sub publish {
      
     unless ($batch) {
      $scrout.='<h3>'.&mt('Metadata Information').' ' .
-       Apache::loncommon::help_open_topic("Metadata_Description")
+       &Apache::loncommon::help_open_topic("Metadata_Description")
        . '</h3>';
     }
 
@@ -1179,9 +1195,18 @@ sub publish {
 #   $env{'from.'..} so that it can be used by the phase two handler in
 #    batch mode
 
-    my $intr_scrout.=
-	'<form name="pubform" action="/adm/publish" method="post">'.
-	'<p>'.($env{'form.makeobsolete'}?'':'<input type="submit" value="'.&mt('Finalize Publication').'" />').'</p>'.
+    my $intr_scrout.='<br />'
+                    .'<form name="pubform" action="/adm/publish" method="post">';
+    unless ($env{'form.makeobsolete'}) {
+       $intr_scrout.='<p>'
+                    .&mt('Searching for your resource will be based on the following metadata. Please provide as much data as possible.')
+                    .'</p>'
+                    .'<p><input type="submit" value="'
+                    .&mt('Finalize Publication')
+                    .'" /></p>';
+    }
+    $intr_scrout.=&Apache::lonhtmlcommon::start_pick_box();
+    $intr_scrout.=
 	&hiddenfield('phase','two').
 	&hiddenfield('filename',$env{'form.filename'}).
 	&hiddenfield('allmeta',&escape($allmeta)).
@@ -1193,10 +1218,7 @@ sub publish {
 	&textfield('Subject','subject',$metadatafields{'subject'});
  # --------------------------------------------------- Scan content for keywords
 
-    my $keywords_help = Apache::loncommon::help_open_topic("Publishing_Keywords");
-    my $KEYWORDS=&mt('Keywords');
-    my $CheckAll=&mt('check all');
-    my $UncheckAll=&mt('uncheck all');
+    my $keywords_help = &Apache::loncommon::help_open_topic("Publishing_Keywords");
     my $keywordout=<<"END";
 <script>
 function checkAll(field) {
@@ -1209,14 +1231,13 @@ function uncheckAll(field) {
         field[i].checked = false ;
 }
 </script>
-<p><font color="#800000" face="helvetica"><b>$KEYWORDS:</b></font>
- $keywords_help</b>
-<input type="button" value="$CheckAll" onclick="javascript:checkAll(document.pubform.keywords)" /> 
-<input type="button" value="$UncheckAll" onclick="javascript:uncheckAll(document.pubform.keywords)" /> 
-</p>
-<br />
 END
-    $keywordout.='<table border="2"><tr>';
+    $keywordout.="\n".&Apache::lonhtmlcommon::row_title(&mt('Keywords'))
+                .$keywords_help
+                .'<input type="button" value="'.&mt('check all').'" onclick="javascript:checkAll(document.pubform.keywords)" />'
+                .'<input type="button" value="'.&mt('uncheck all').'" onclick="javascript:uncheckAll(document.pubform.keywords)" />'
+                .'</p><br />'
+                .'<table border="2"><tr>';
     my $colcount=0;
 
     foreach (sort keys %keywords) {
@@ -1239,7 +1260,8 @@ END
     }
     $env{'form.keywords'}=~s/\,$//;
 
-    $keywordout.='</tr></table>';
+    $keywordout.='</tr></table>'
+                 .&Apache::lonhtmlcommon::row_closure();
 
     $intr_scrout.=$keywordout;
 
@@ -1247,28 +1269,24 @@ END
 
     $intr_scrout.=&textfield('Notes','notes',$metadatafields{'notes'});
 
-    $intr_scrout.=
-	"\n<p><font color=\"#800000\" face=\"helvetica\"><b>".&mt('Abstract').":".
-	"</b></font></p><br />".
-	'<textarea cols="80" rows="5" name="abstract">'.
-	$metadatafields{'abstract'}.'</textarea></p>';
+    $intr_scrout.="\n".&Apache::lonhtmlcommon::row_title(&mt('Abstract'))
+                 .'<textarea cols="80" rows="5" name="abstract">'
+                 .$metadatafields{'abstract'}
+                 .'</textarea>'
+                 .&Apache::lonhtmlcommon::row_closure();
 
     $source=~/\.(\w+)$/;
 
+    $intr_scrout.="\n".&Apache::lonhtmlcommon::row_title(&mt('Grade Levels'))
+                 .&mt('Lowest Grade Level:').'&nbsp;'
+                 .&select_level_form($metadatafields{'lowestgradelevel'},'lowestgradelevel')
+#                .&Apache::lonhtmlcommon::row_closure();
+#   $intr_scrout.="\n".&Apache::lonhtmlcommon::row_title(&mt('Highest Grade Level'))
+                 .' '.&mt('Highest Grade Level:').'&nbsp;'
+                 .&select_level_form($metadatafields{'highestgradelevel'},'highestgradelevel')
+                 .&Apache::lonhtmlcommon::row_closure();
 
-    $intr_scrout.=
-	"\n<p><font color=\"#800000\" face=\"helvetica\"><b>".
-	&mt('Lowest Grade Level').':'.
-	"</b></font></p><br />".
-	&select_level_form($metadatafields{'lowestgradelevel'},'lowestgradelevel').
-	"\n<p><font color=\"#800000\" face=\"helvetica\"><b>".
-	&mt('Highest Grade Level').':'.
-	"</b></font></p><br />".
-	&select_level_form($metadatafields{'highestgradelevel'},'highestgradelevel').
-	&textfield('Standards','standards',$metadatafields{'standards'});
-
-
-
+    $intr_scrout.=&textfield('Standards','standards',$metadatafields{'standards'});
 
     $intr_scrout.=&hiddenfield('mime',$1);
 
@@ -1327,23 +1345,22 @@ END
 				     (grep !/^priv$/,(&Apache::loncommon::copyrightids)));
 	}
 	my $copyright_help =
-	    Apache::loncommon::help_open_topic('Publishing_Copyright');
-	$intr_scrout =~ s/Distribution:/'Distribution: ' . $copyright_help/ge;
-	$intr_scrout.=&text_with_browse_field('Custom Distribution File','customdistributionfile',$metadatafields{'customdistributionfile'},'rights').$copyright_help;
+	    &Apache::loncommon::help_open_topic('Publishing_Copyright');
+        my $replace=&mt('Copyright/Distribution:');
+	$intr_scrout =~ s/$replace/$replace.' '.$copyright_help/ge;
+
+	$intr_scrout.=&text_with_browse_field('Custom Distribution File','customdistributionfile',$metadatafields{'customdistributionfile'},'rights');
 	$intr_scrout.=&selectbox('Source Distribution','sourceavail',
 				 $defaultsourceoption,
 				 \&Apache::loncommon::source_copyrightdescription,
 				 (&Apache::loncommon::source_copyrightids));
 #	$intr_scrout.=&text_with_browse_field('Source Custom Distribution File','sourcerights',$metadatafields{'sourcerights'},'rights');
 	my $uctitle=&mt('Obsolete');
-	$intr_scrout.=
-	    "\n<p><label><font color=\"#800000\" face=\"helvetica\"><b>$uctitle:".
-	    '</b></font> <input type="checkbox" name="obsolete" ';
-	if ($metadatafields{'obsolete'}) {
-	    $intr_scrout.=' checked="1" ';
-	}
-	$intr_scrout.='/ ></label></p>'.
-	    &text_with_browse_field('Suggested Replacement for Obsolete File',
+        my $obsolete_checked=($metadatafields{'obsolete'})?' checked="1" ':'';
+        $intr_scrout.="\n".&Apache::lonhtmlcommon::row_title($uctitle)
+                     .'<input type="checkbox" name="obsolete" '.$obsolete_checked.'/ >'
+                     .&Apache::lonhtmlcommon::row_closure(1);
+        $intr_scrout.=&text_with_browse_field('Suggested Replacement for Obsolete File',
 				    'obsoletereplacement',
 				    $metadatafields{'obsoletereplacement'});
     } else {
@@ -1374,8 +1391,12 @@ END
 				    $metadatafields{'obsoletereplacement'});
    }
     if (!$batch) {
-	$scrout.=$intr_scrout.'<p><input type="submit" value="'.
-	    &mt($env{'form.makeobsolete'}?'Make Obsolete':'Finalize Publication').'" /></p></form>';
+	$scrout.=$intr_scrout
+            .&Apache::lonhtmlcommon::end_pick_box()
+            .'<p><input type="submit" value="'
+	    .&mt($env{'form.makeobsolete'}?'Make Obsolete':'Finalize Publication')
+            .'" /></p>'
+            .'</form>';
     }
     return($scrout,0);
 }
@@ -1577,7 +1598,7 @@ sub phasetwo {
         unless ($srcd=~/^\/home\/httpd\/html\/res/) {
             print $logfile "\nPANIC: Target dir is ".$srcd;
             $r->print(
-	 "<span class=\"LC_error\">Invalid target directory, FAIL</span>");
+	 "<span class=\"LC_error\">".&mt('Invalid target directory, FAIL')."</span>");
 	    return 0;
         }
         opendir(DIR,$srcd);
@@ -1699,7 +1720,7 @@ sub phasetwo {
         $r->print(
            '<hr /><a href="'.$thisdistarget.'"><font size="+2">'.
            &mt('View Published Version').'</font></a>'.
-           '<p><a href="'.$thissrc.'"><font size=+2>'.
+           '<p><a href="'.$thissrc.'"><font size="+2">'.
 		  &mt('Back to Source').'</font></a></p>'.
            '<p><a href="'.$thissrcdir.
                    '"><font size="+2">'.
@@ -1744,6 +1765,7 @@ sub notify {
 	print $logfile "\n============ Done ============\n";
 	$logfile->close();
     }
+    if ($lock) { &Apache::lonnet::remove_lock($lock); }
     return OK;
 }
 
@@ -1813,7 +1835,9 @@ sub publishdirectory {
                   &checkbox('obsolete','make file(s) obsolete').
 		  &checkbox('forceoverride','force directory level catalog information over existing').
 		  '<br /><input type="submit" value="'.&mt('Publish Directory').'" /></form>');
+        $lock=0;
     } else {
+        unless ($lock) { $lock=&Apache::lonnet::set_lock(&mt('Publishing [_1]',$fn)); }
 # actually publish things
 	opendir(DIR,$fn);
 	my @files=sort(readdir(DIR));
@@ -2099,28 +2123,57 @@ sub handler {
 	my $thistype=$1;
 	my $thisembstyle=&Apache::loncommon::fileembstyle($thistype);
         if ($thistype eq 'page') {  $thisembstyle = 'rat'; }
-	$r->print('<h2>'.&mt('Publishing').' '.
-		  &Apache::loncommon::filedescription($thistype).' <tt>');
 
+	$r->print('<h2>'.&mt('Publishing [_1]','<tt>'.$thisdisfn.'</tt>').'</h2>');
+
+        $r->print('<h3>'.&mt('Resource Details').'</h3>');
+
+        $r->print(&Apache::lonhtmlcommon::start_pick_box());
+
+        $r->print(&Apache::lonhtmlcommon::row_title(&mt('Type'))
+                 .&Apache::loncommon::filedescription($thistype)
+                 .&Apache::lonhtmlcommon::row_closure()
+                 );
+
+        $r->print(&Apache::lonhtmlcommon::row_title(&mt('Link to Resource'))
+                 .'<tt>'
+                 );
 	$r->print(<<ENDCAPTION);
 <a href='javascript:void(window.open("/~$cuname/$thisdisfn","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
 $thisdisfn</a>
 ENDCAPTION
-        $r->print('</tt></h2><b>'.&mt('Target').':</b> <tt>'.
-		  $thisdistarget.'</tt><br />');
-   
+        $r->print('</tt>'
+                 .&Apache::lonhtmlcommon::row_closure()
+                 );
+
+        $r->print(&Apache::lonhtmlcommon::row_title(&mt('Target'))
+                 .'<tt>'.$thisdistarget.'</tt>'
+                 .&Apache::lonhtmlcommon::row_closure()
+                 );
+
+# SB - ToDo:   
 	if (($cuname ne $env{'user.name'})||($cudom ne $env{'user.domain'})) {
-	    $r->print('<h3><font color="red">'.&mt('Co-Author').': '.
-		      $cuname.&mt(' at ').$cudom.'</font></h3>');
+#           $r->print(&Apache::lonhtmlcommon::row_title('<span class="LC_warning">'.&mt('Co-Author').'</span>')
+            $r->print(&Apache::lonhtmlcommon::row_title(&mt('Co-Author'))
+                     .'<span class="LC_warning">'
+		     .&mt('[_1] at [_2]',$cuname,$cudom)
+                     .'</span>'
+                     .&Apache::lonhtmlcommon::row_closure()
+                     );
 	}
 
 	if (&Apache::loncommon::fileembstyle($thistype) eq 'ssi') {
+            $r->print(&Apache::lonhtmlcommon::row_title(&mt('Diffs')));
 	    $r->print(<<ENDDIFF);
-<br />
 <a href='javascript:void(window.open("/adm/diff?filename=/~$cuname/$thisdisfn&versiontwo=priv","cat","height=300,width=500,scrollbars=1,resizable=1,menubar=0,location=1"))'>
 ENDDIFF
-            $r->print(&mt('Diffs with Current Version').'</a><br />');
+            $r->print(&mt('Diffs with Current Version').'</a>'
+                     .&Apache::lonhtmlcommon::row_closure()
+                     );
+
 	}
+
+        $r->print(&Apache::lonhtmlcommon::end_pick_box());
   
 # ------------------ Publishing from $thisfn to $thistarget with $thisembstyle.
 
@@ -2135,7 +2188,7 @@ ENDDIFF
 		    &publish($thisfn,$thistarget,$thisembstyle);
 		$r->print('<hr />'.$outstring);
 	    } else {
-		$r->print('<h3>'.
+		$r->print('<h3 class="LC_error">'.
 			  &mt('The document contains errors and cannot be published.').
 			  '</h3>');
 	    }
