@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Construct and maintain state and binary representation of course for user
 #
-# $Id: lonuserstate.pm,v 1.127 2007/12/01 03:43:49 albertel Exp $
+# $Id: lonuserstate.pm,v 1.128 2008/03/12 02:45:50 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -574,7 +574,7 @@ sub accinit {
     my $courseuri=$uri;
     $courseuri=~s/^\/res\///;
     &Apache::lonnet::delenv('(acc\.|httpref\.)');
-    &Apache::lonnet::appenv(%acchash);
+    &Apache::lonnet::appenv(\%acchash);
 }
 
 # ---------------- Selectively delete from randompick maps and hidden url parms
@@ -630,7 +630,7 @@ sub hiddenurls {
     }
 # --------------------------------------- append randomout entry to environment
     if ($randomoutentry) {
-	&Apache::lonnet::appenv('acc.randomout' => $randomoutentry);
+	&Apache::lonnet::appenv({'acc.randomout' => $randomoutentry});
     }
 }
 
@@ -678,9 +678,9 @@ sub readmap {
 	$hash{'is_map_0.0'}=1;
 	loadmap($uri,'0.0');
 	if (defined($hash{'map_start_'.$uri})) {
-	    &Apache::lonnet::appenv("request.course.id"  => $short,
-				    "request.course.fn"  => $fn,
-				    "request.course.uri" => $uri);
+	    &Apache::lonnet::appenv({"request.course.id"  => $short,
+				     "request.course.fn"  => $fn,
+				     "request.course.uri" => $uri});
 	    $env{'request.course.id'}=$short;
 	    &traceroute('0',$hash{'map_start_'.$uri},'&');
 	    &accinit($uri,$short,$fn);
@@ -751,9 +751,9 @@ sub readmap {
 		(tie(%hash,'GDBM_File',"$fn.db",&GDBM_READER(),0640))) {
 		if (tie(%parmhash,'GDBM_File',$fn.'_parms.db',&GDBM_READER(),0640)) {
 		    $retfurl='/adm/navmaps';
-		    &Apache::lonnet::appenv("request.course.id"  => $short,
-					    "request.course.fn"  => $fn,
-					    "request.course.uri" => $uri);
+		    &Apache::lonnet::appenv({"request.course.id"  => $short,
+					     "request.course.fn"  => $fn,
+					     "request.course.uri" => $uri});
 		    untie(%hash);
 		    untie(%parmhash);
 		    last;
@@ -814,7 +814,7 @@ sub evalstate {
 	    }
 	}
     }
-    &Apache::lonnet::appenv('user.state.'.$env{'request.course.id'} => $state);
+    &Apache::lonnet::appenv({'user.state.'.$env{'request.course.id'} => $state});
     return $state;
 }
 
