@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Wrapper for external and binary files as standalone resources
 #
-# $Id: lonwrapper.pm,v 1.27 2008/01/15 15:23:28 www Exp $
+# $Id: lonwrapper.pm,v 1.28 2008/03/03 19:57:28 www Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -68,7 +68,11 @@ sub wrapper {
 
     my %layout = ('border' => 0);
     if ($env{'environment.remote'} eq 'off') {
-	$layout{'rows'} = "200,*";
+        if ($env{'environment.icons'} eq 'iconsonly') {
+           $layout{'rows'} = "100,*";
+        } else {
+	   $layout{'rows'} = "200,*";
+        }
     } else {
 	$layout{'rows'} = "1,*";
 	$topurl = "/adm/rat/empty.html";
